#include "AnalysisAlgorithmMap.h"
#include "AnalysisAlgorithm.h"
#include "CalibrationController/IS/TestData.h"

namespace SctAnalysis {

AnalysisAlgorithmMap& AnalysisAlgorithmMap::instance() {
    static AnalysisAlgorithmMap* instance = new AnalysisAlgorithmMap();
    return *instance;
}
    
shared_ptr <AnalysisAlgorithm> AnalysisAlgorithmMap::getAlgorithm(const TestData& testData, const string& moduleName) const throw(Sct::InvalidArgument) {
    if (algMap.count(testData.testName) == 0) {
	Sct::InvalidArgumentError iae("No map for Test: " + testData.testName, __FILE__, __LINE__);
	iae.setSeverity(MRS_DIAGNOSTIC);
	throw iae;
    }
    return (*algMap.find(testData.testName)).second->clone(testData, moduleName);
}
    
bool AnalysisAlgorithmMap::setAlgorithm(const string testName, auto_ptr<AnalysisAlgorithm> alg) throw() {
    algMap[testName] = shared_ptr<AnalysisAlgorithm>(alg);
    return true;    
}

    
}
