#include "ConfigUpdater.h"
#include "ConfigUpdaterManager.h"
#include "Sct/SctParameters.h"
#include "Sct/SctNames.h"
#include "SctData/TrimRangeTestResult.h"
#include "SctData/StrobeDelayTestResult.h"
#include "SctData/NPtGainTestResult.h"
#include "SctData/ResponseCurve.h"
#include "CalibrationControllerImpl.h"
#include "CommonWithDsp/ABCD/ABCDscans.h"

using SctData::TestResult;
using namespace Sct;

namespace SctCalibrationController {
    
    unsigned long ConfigUpdater::getMID(const SctData::TestResult& t) const {
	Sct_SctApiStatus status;
	///@todo remove the nasty const_cast below when proper CORBA implementation arrives.
	    return CalibrationControllerImpl::instance().getApi()->findModuleSerial(&status, const_cast<char*> ( t.getModuleName().c_str())  );
    }
    
    bool StrobeDelayConfigUpdater::inMap = ConfigUpdaterManager::instance().setUpdater("SctData::StrobeDelayTestResult", shared_ptr<ConfigUpdater> (new StrobeDelayConfigUpdater()));
    
    void StrobeDelayConfigUpdater::update(const TestResult& testresult) const {
	const SctData::StrobeDelayTestResult& t = dynamic_cast<const SctData::StrobeDelayTestResult&> (testresult);
        unsigned long mid = getMID(t); 
	SctNames::Mrs() << "CC_UPDATE" << MRS_TEXT("CalibrationController updating StrobeDelay") << MRS_PARAM<long>("ModuleID", mid) << MRS_DIAGNOSTIC << ENDM; 
        for (unsigned ichip=0; ichip<Sct::nChipModule; ++ichip) {
            Sct_SctApiStatus status;
            CalibrationControllerImpl::instance().getApi()->modifyABCDChipVar(&status, mid, ichip, ST_STROBE_DELAY, (double) t.getOptimum(ichip));
        } 
    }
    
    bool TrimRangeConfigUpdater::inMap = ConfigUpdaterManager::instance().setUpdater("SctData::TrimRangeTestResult", shared_ptr<TrimRangeConfigUpdater> (new TrimRangeConfigUpdater() ) );
    
    void TrimRangeConfigUpdater::update(const TestResult& testresult) const {
	const SctData::TrimRangeTestResult& t = dynamic_cast<const SctData::TrimRangeTestResult&> (testresult) ;
	unsigned long mid = getMID(t);
	short trims[nChannelModule];
	for (unsigned ichip=0; ichip<Sct::nChipModule; ++ichip) {
            Sct_SctApiStatus status;
	    CalibrationControllerImpl::instance().getApi()->
		modifyABCDChipVar(&status, mid, ichip, ST_TRIM_RANGE , (double) t.chipTrim[ichip]->range );
	    for (unsigned ichannel=0; ichannel < nChannelChip; ++ ichannel){
		trims[ichip*nChannelChip+ichannel] =  t.chipTrim[ichip]->channelTrim.getAt(ichannel).value.trim ;
	    }
	}
	SctNames::Mrs() << "CC_UPDATE" << MRS_TEXT("CalibrationController updating trims") << MRS_PARAM<long>("ModuleID", mid) << MRS_INFORMATION << ENDM;
        Sct_SctApiStatus status;
	Sct_SctApi_T_CharBlock data = _Sct_SctApi_T_CharBlock_sequence::Create(nChannelModule, trims);
	CalibrationControllerImpl::instance().getApi()->modifyABCDTrims(&status, mid, data) ;
    }

    bool NPtGainConfigUpdater::inMap = ConfigUpdaterManager::instance().setUpdater("SctData::NPtGainTestResult", shared_ptr<NPtGainConfigUpdater> (new NPtGainConfigUpdater() ) );
    
    void NPtGainConfigUpdater::update(const TestResult& testresult) const {
	const SctData::NPtGainTestResult& t = dynamic_cast<const SctData::NPtGainTestResult&> (testresult) ;
        unsigned long mid = getMID(t);
	SctNames::Mrs() << "CC_UPDATE" << MRS_TEXT("CalibrationController updating response curve") << MRS_PARAM<const char*>("Module", t.getModuleName().c_str()) << MRS_INFORMATION << ENDM; 
        for (unsigned ichip=0; ichip<Sct::nChipModule; ++ichip) {
	    shared_ptr<const SctData::ResponseCurve> rc = t.getChipData(ichip).rc;
	    shared_ptr<const TF1> func = rc->getFunction();
/*	    SctNames::Mrs() << "CC_UPDATE_INFO" << MRS_TEXT("CalibrationController updating response curve") << MRS_PARAM<long>("ModuleID", mid) << MRS_PARAM<int>("Chip", ichip) 
		            << MRS_PARAM<int>("Function", rc->getIndex()) << MRS_PARAM<double>("Param0", (double) func->GetParameter(0)) 
			    << MRS_PARAM<double>("Param1", (double) func->GetParameter(1)) << MRS_PARAM<double>("Param2", (double) func->GetParameter(2)) 
			    << MRS_DIAGNOSTIC << ENDM;*/
	    Sct_SctApiStatus status;
            CalibrationControllerImpl::instance().getApi()->
		modifyABCDRC(&status, mid, ichip, rc->getIndex(), 
			     (double) func->GetParameter(0), 
			     (double) func->GetParameter(1), 
			     (double) func->GetParameter(2)    );
        }
	
    }
}
