#ifndef SCT_CALIBRATIONCONTROL_CONFIGUPDATER
#define SCT_CALIBRATIONCONTROL_CONFIGUPDATER
#include <boost/noncopyable.hpp>
#include "SctData/TestResult.h"

namespace SctCalibrationController {

/**
   Algorithm interface for updating the module configuration after the 
   result of a test (TestResult) becomes available.
   Concrete sub-classes must impliment the update() method.
   Applies changes via the IPC interface of the SctApi.
*/

class ConfigUpdater : boost::noncopyable {
public:
    virtual void update(const SctData::TestResult& t) const = 0;
protected:
    ConfigUpdater(){;}
    virtual ~ConfigUpdater(){;}
    unsigned long getMID(const SctData::TestResult& t) const;
};


class StrobeDelayConfigUpdater : public ConfigUpdater{
public:
    virtual void update(const SctData::TestResult& t) const;

    /// Do not delete!
    virtual ~StrobeDelayConfigUpdater() {;}
private:
    StrobeDelayConfigUpdater() {;}
    static bool inMap;
};

class NPtGainConfigUpdater : public ConfigUpdater{
public:
    virtual void update(const SctData::TestResult& t) const;
private:
    NPtGainConfigUpdater() {;}
    static bool inMap;
};

class TrimRangeConfigUpdater : public ConfigUpdater{
public:
    virtual void update(const SctData::TestResult& t) const;
private:
    TrimRangeConfigUpdater() {;}
    static bool inMap;
};

} // end of namespace CallibrationController

#endif
