#ifdef linux
#undef linux
#endif

#include "RunController.h"
#include "CalibrationControllerImpl.h"
#include "ipc/CalibrationController.h"
#include "Sct/SctNames.h"
#include "Sct/Exception.h"
#include <rc/RunParamsInfo.h>
#include <ipc/partition.h>
#include <is/isinfo.h>
#include <iostream>
#include <fstream>
#include <ilu/iluxport.h>

using namespace std;
using namespace Sct;

int main(int argc, char** argv) {
    using namespace SctCalibrationController;
   
    cout << "Starting Calibration controller" << endl;
    setExceptionHandlers(argv[0]);
    
    IPCCore::init(true);
    //IPCPartition p("SCT");
    
    RunControl& rc = RunControl::instance();
    cout << "Instance created" << endl;
        
    rc.rc_start("CalibrationController", "SCT");
    cout << "All over" << endl;

    return 0;
}

namespace SctCalibrationController {
    
RunControl::RunControl() : cc (CalibrationControllerImpl::initialize(*this)) {	
}
    
RunControl& RunControl::instance() {
    static RunControl rc;
    return rc;
}


int RunControl::loadaction () {
    cout << "Loadaction" << endl;
    api = (Sct_SctApi_T_SctApi*)ipcPartition().lookup("Test_Server", "SCTSctApiServer", Sct_SctApi_T_SctApi::ILUClassRecord);
    if (!api) {
	SctNames::Mrs() << "RC_INIT" << MRS_TEXT("RunController failed to find SctApi") << MRS_ERROR << ENDM;
	return 1;
    }
    cc.setApi(*api);
    Sct_SctApiStatus s;
    api->initialiseAll(&s, 0);
    
    
    return 0;
}

void RunControl::enterConfigured() {
    Sct_SctApiStatus status;
    api->configureAllModules(&status);
}

void RunControl::enterRunning() {
    if (!api) {
	cout << "Can't enter running as we don't have the api" << endl;
	return;
    }
    cout << "enter Running"  << endl;
    Sct_SctApiStatus status;
    ///@todo Check status
    RunParamsInfo runParams;
    ISInfoDictionary& is = SctNames::getISDictionary();
    ISInfo::Status isStatus;

    isStatus = is.findValue("RunParams.RunParams", runParams);
    api->setRunNumber(&status, runParams.run_number);
    api->setScanNumber(&status, 0);
    cout << "New Run, setting run number and scan number: " << runParams.run_number << "  " << 0 << endl;
    cc.takeControl(runParams.run_number);
}

void RunControl::exitRunning() {
    Sct_SctApiStatus status;
    cout << "exit Running" << endl;
    cc.giveupControl();
}

int RunControl::unloadaction () {
    cout << "unloadaction" << endl;

    Sct_SctApiStatus status;

    if (api) api->shutdownAll(&status);

    return 0;
}

int RunControl::resetaction () {
    cc.reset();
    return 0;
}

void RunControl::sctApiError(ErrorState state, const string& reason) {
    this->setError(state, reason);
    api = 0;
    cc.giveupControl();
}

}
