#ifndef TRIMRANGETEST_H
#define TRIMRANGETEST_H

#include "../ipc/TestRequest.h"
#include "../ipc/CalibrationController.h"
#include "../ipc/ScanLibrary.h"
#include "../ipc/Sct_CalibrationController.hh"
#include "../CalibrationControllerImpl.h"

#include "../SctApiException.h"
#include "Sct/LogicErrors.h"

#include <CommonWithDsp/ABCD/ABCDscans.h>
#include <iostream>
#include <vector>

using namespace std;
using namespace SctCalibrationController::Ipc;
using namespace SctCalibrationController;
using namespace Sct;

namespace SctCalibrationController {
namespace Scripts {
    
class TrimRangeTest : public TestRequestImpl {
public:	
    static TrimRangeTest& instance() {
	static TrimRangeTest tr;
	return tr;
    }
    
    virtual void startTest(Sct_SctApi_T_SctApi& inapi, string& name, unsigned short& testVariable, unsigned long& nScans, vector<double>& testPoints) {	
	this->api = &inapi;
	
	Sct_SctApiStatus status;
	api->modifyABCDVar(&status, ST_COMPRESSION, 1); // X1X compression
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "TrimRangeTest::startTest can't set compression", __FILE__, __LINE__);
	
	api->modifyABCDVar(&status, ST_EDGE_DETECT, 0); // edge detect off
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "TrimRangeTest::startTest can't set edge detect", __FILE__, __LINE__);
	
	api->modifyABCDVar(&status, ST_SEND_MASK, 0); // route mask off
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "TrimRangeTest::startTest can't set send mask", __FILE__, __LINE__);
	
	api->modifyABCDVar(&status, ST_QCAL, 1);   // QCAL (fC) 
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "TrimRangeTest::startTest can't set qcal", __FILE__, __LINE__);
	
	name = "TrimRangeTest";	
	testVariable=ST_TRIM_RANGE;
	for (unsigned int i=0; i<28; ++i) {
	    testPoints.push_back(getValue(i));
	}
	nScans = testPoints.size();
    }
    
    virtual Sct_SctApi_T_Scan* getNextScan(unsigned long index) {
	Sct_SctApiStatus status;    
            
	if (isFirstPoint(index)) {
	    api->modifyABCDVar(&status, ST_TRIM_RANGE, getRange(index)); // set trim range
	    if (status.returnCode != 0)
		throw SctApiException(status.returnCode, "TrimRangeTest::getNextScan", __FILE__, __LINE__);
	}
	api->modifyABCDVar(&status, ST_TRIM, getValue(index));                       // set trim value
	if (status.returnCode != 0)
	    throw SctApiException(status.returnCode, "TrimRangeTest::getNextScan", __FILE__, __LINE__);

	
	ScanLibrary& sl = CalibrationControllerImpl::instance().getScanLibrary();
	return &sl.thresholdScanfC(1000, 1.0);  // trim at 1 fC
    }
    
    unsigned int getRange(unsigned int index) {
	if (index<16) return 0;
	else return (index-16)/4+1;
    }
    
    bool isFirstPoint(unsigned int index) {
	return (index-16)%4 == 0;
    }
    
    unsigned int getValue(unsigned int index) {
	if (index < 16) return index;
	unsigned int pointInRange = (index-16)%4;
	return (pointInRange+1)*4-1;
    }
    
    virtual bool canFeedback() {
	return true;
    }    

    virtual void endTest() {
	api = 0;
    }
        
private:
    TrimRangeTest() {}
    Sct_SctApi_T_SctApi* api;
};
}
}
#endif //TRIMRANGETEST_H
