#include "FitterWorkerGroup.h"
#include "Fitter.h"
#include "FitAlgorithm.h"
#include "FitAlgorithmMap.h"

#include "Sct/SctNames.h"
#include "Sct/IS/IOManagerIS.h"
#include "Sct/ISProxy/IOManagerISProxy.h"
#include "Sct/IS/IONameIS.h"
#include "SctData/FitScanResult.h"
#include "SctData/RawScanResult.h"
#include "Sct/Timer.h"
#include "Sct/StdExceptionWrapper.h"

using namespace std;
using namespace SctService;
using boost::shared_ptr;
using SctData::RawScanResult;

namespace SctFitter {


void FitterWorkerGroup::work(shared_ptr<const RawScanResult> raw) throw() {
    using namespace SctData;
    double start = timer();
    
    try {
        // make a cache of the name of this scan:
        Fitter::instance().setLastScanName(raw->getUniqueID().c_str() );
        Fitter::instance().incrementFitsDone();
        m_calls++;

        std::cout << "This is call number  " << m_calls << " to this worker. Working on " << raw->getUniqueID().c_str() << " Q = "<< queueSize()<< std::endl;

        const FitAlgorithm& alg = FitAlgorithmMap::instance().getAlgorithm((*raw).getHeader().getVariable());
        std::auto_ptr<SctData::FitScanResult> fitted = alg.doFit(*raw);

        //Publish object
        Sct::IS::IOParamsIS params(Sct::SctNames::getFittedDataName());
        Sct::ISProxy::IOManagerISProxy::instance().write(*fitted, &params);	
	
    } catch (Sct::InvalidArgumentError& e) {
        e.sendToMrs();
        Fitter::instance().incrementFitErrors();
    } catch(Sct::Throwable& e) {
        e.sendToMrs(MRS_ERROR);
        Fitter::instance().incrementFitErrors();
    } catch(std::exception& e) {
	StdExceptionWrapper sew(e);
        sew.sendToMrs(MRS_ERROR);
        Fitter::instance().incrementFitErrors();	
    } catch(...) {
	Error e("uncaught unknown exception", __FILE__, __LINE__);
	e.sendToMrs(MRS_ERROR);
	Fitter::instance().incrementFitErrors();	
    }
    Fitter::instance().scanDone(timer()-start);
}

}// end of namespace SctFitter
