#include "IOManager.h"
#include "Streamer.h"
#include "Streamable.h"
#include "Serializable.h"
#include "IStream.h"
#include "OStream.h"
#include "SctNames.h"
#include <sstream>

using namespace std;

namespace Sct {

typedef map<string, shared_ptr<Streamer> > StreamerMap;
    
bool IOManager::addToMap(const string& className, std::auto_ptr<Streamer> s) throw() {
    StreamerMap& map = getMap();
    if (map.find(className) != map.end())
        return false;
    map[className] = s;
    return true;
}

StreamerMap& IOManager::getMap() throw() {
    static StreamerMap map;
    return map;
}

Streamer& IOManager::getStreamer(const string& className) const {
    StreamerMap& map = getMap();
    if (map.find(className) == map.end()) {
	ostringstream oss;		
	oss << "Known streamers:" << endl;
	for (StreamerMap::const_iterator it=map.begin();
	     it!=map.end();
	     ++it){
	    oss << (*it).first << endl; 
	}
	string diag = oss.str();
	SctNames::Mrs() << "SCT_IO" << MRS_TEXT(diag) << MRS_DIAGNOSTIC << ENDM;
	throw NoSuchStreamerException(className, "IOManager can't find a Streamer", __FILE__, __LINE__);
    }
    return *(*map.find(className)).second;
}

void IOManager::writeImpl(OStream& out, const Streamable& ob, bool bWriteClassName) const {
    Streamer& s = getStreamer(ob.getClassName());
    if (bWriteClassName) writeClassName(out, ob.getClassName());
    s.write(out, ob, *this);
}

shared_ptr<Streamable> IOManager::readImpl(IStream& in) const {
    string className = readClassName(in);
    Streamer& s = getStreamer(className);
    return s.read(in, *this);
}

shared_ptr<Streamable> IOManager::readImpl(IStream& in, const string& className) const {
    Streamer& s = getStreamer(className);
    return s.read(in, *this);    
}

void IOManager::readImpl(IStream& in, Streamable& ob, bool bReadClassName) const {
    if (bReadClassName) {
	string className = readClassName(in);
	if (className != ob.getClassName()) {
	    string str = "Can't read: ";
	    str += ob.getClassName();
	    str += " from stream because stream contains a: ";
	    str += className;
	    throw StreamCorruptedException(str, __FILE__, __LINE__);
	}
    }
    Streamer& s = getStreamer(ob.getClassName());
    s.read(in, ob, *this);
}

void IOManager::writeClassName(OStream& out, const string& className) const {
    out << "SctClassName:" + className;
}

string IOManager::readClassName(IStream& in) const {
    string className = "";
    in >> className;
    if (className.find("SctClassName:") == 0) {
	return className.substr(13);
    } 
    throw StreamCorruptedException("Read string but it wasn't a className.  Got: " + className, __FILE__, __LINE__);
}

    

}
