#include "ErfcFitObject.h"
#include "TF1.h"

using namespace Sct;

namespace SctData {

ErfcFitObject::ErfcFitObject() throw() {
    this->init();
    this->reset();
}

auto_ptr<FitObject> ErfcFitObject::clone() const throw() {
	return auto_ptr<FitObject> (new ErfcFitObject(*this));
}

ErfcFitObject& ErfcFitObject::operator=(const TF1& f)throw(LogicError)  {
    this->FitObject::operator=(f);
    return *this;
}

ErfcFitObject::ErfcFitObject(const TF1& f) throw(LogicError) {
    this->init();
    this->reset();
    *this=f;
}

// make a basic ErfcFitObject
void ErfcFitObject::init() throw() {
    this->setFormula("erfc");
    this->setNDim(1);
    this->setNPar(3);
    //Catch LogicError which can't ever be thrown!
    try {
        this->setParName(0,"Prefact");
        this->setParName(1,"Mean");
        this->setParName(2,"Sigma");
    } catch (LogicError&) {}
}

// Call the appropriate constructor for making a ROOT TF1 object from this ErfFitObject
auto_ptr<TF1> ErfcFitObject::makeBasicRootTF1() const throw(LogicError) {
    auto_ptr<TF1> r( new TF1("myerfc", SctData::erfc, this->getVarMin(0), this->getVarMax(0), 3));
    if (!r.get())
        throw InvariantViolatedError("ErfcFitObject::makeBasicRootTF1()", __FILE__, __LINE__);
    return r;
}

} // end of namespace SctData

