#ifndef ERFCFITOBJECT_H
#define ERFCFITOBJECT_H

#include "TMath.h"
#include "FitObject.h"
#include "FitFunctions.h"

namespace SctData{

/** 
 * A derived class of the FitObject class of complimentary error function
 * <P>
 * parameters: prefactor, mean, sigma. </P>
 * Evaluation uses TMath function Erf(x) = (2/sqrt(pi)) Integral(exp(-t^2))dt between 0 and x
 * @author Alan Barr
 * @date 9 December 2002
 */

class ErfcFitObject : public FitObject {
public:
    /// Constructor 
    ErfcFitObject() throw() ;
    
    /// Constructor from a root TF1;.
    ErfcFitObject(const TF1& rootfunc) throw(LogicError);

    virtual string getClassName() const throw() {return "SctData::ErfcFitObject";}
    
    virtual auto_ptr<FitObject> clone() const throw();

    /// Destructor does nothing
    virtual ~ErfcFitObject() throw() {;}
	  
    /// equality with root TF1
    ErfcFitObject& operator=(const TF1& rootfunc) throw(LogicError);
	  
protected:
    /// call the correct constructor for making a ROOT TF1
    auto_ptr<TF1> makeBasicRootTF1() const throw(LogicError);

    /// initialisation method called in constructor.
    void init() throw();

}; // end of definition of class ErfcFitObject
    
}// end of namespace SctData
#endif // #ifndef ERFCFITOBJECT_H
