#include "FitObjectStreamer.h"
#include "../FitObject.h"
#include "../AllFitObject.h"
#include <is/isinfo.h>

using namespace Sct;

namespace SctData {
namespace IO {

void FitObjectStreamer::write(OStream& out, const Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {
    //Dynamic cast should always be safe
    const FitObject& fo = dynamic_cast<const FitObject&>(ob);

    //out << fo.getNPar() << fo.getNDim();
    out << fo.getChiSquared() << fo.getNDF();
    //out << fo.getFormula();
    for (int i=0; i<fo.getNPar(); ++i) {
        out << fo.getParameter(i) << fo.getParError(i) << fo.isFixed(i);
        //out << fo.getParName(i);
    }

    for (int i=0; i<fo.getNDim(); ++i) {
    	out << fo.getVarMin(i) << fo.getVarMax(i);
    }
}

///@todo Think about this- should we set vars that are already initilized??  For now, yes
void FitObjectStreamer::read(IStream& in, Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {
    //Dynamic cast should always be safe
    FitObject& fo = dynamic_cast<FitObject&>(ob);
    helper.set(fo);

    int npar, ndim;
    int ndf;
    double chi2;
    bool fixed;
    //string formula;
    //in >> npar >> ndim;
    ndim = fo.getNDim();
    npar = fo.getNPar();
    in >> chi2 >> ndf;
    //in >> formula;
    //helper.setNDim(ndim);
    //helper.setNPar(npar);
    helper.setChiSquared(chi2);
    helper.setNDF(ndf);
    //helper.setFormula(formula);

    for (int i=0; i<npar; i++) {
        double param, paramError;
        //string name;
        in >> param >> paramError >> fixed;
	//in >> name;
        fo.setParameter(i, param);
        fo.setParError(i, paramError);
	fo.fixParameter(i, fixed);
        //fo.setParName(i, name);
    }

    
    for (int i=0; i<ndim; i++) {
        double varMin, varMax;
        in >> varMin >> varMax;
        fo.setVarMin(i, varMin);
        fo.setVarMax(i, varMax);
    }
}

bool ErfcFitObjectStreamer::inMap = IOManager::addToMap("SctData::ErfcFitObject", auto_ptr<Streamer>(new ErfcFitObjectStreamer()));

shared_ptr<Streamable> ErfcFitObjectStreamer::read(IStream& in, const IOManager& manager) const throw(LogicError, IoError) {
    shared_ptr<Streamable> f (new ErfcFitObject());
    read(in, *f, manager);
    return f;
}

void ErfcFitObjectStreamer::read(IStream& in, Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {
    FitObjectStreamer::read(in, ob, manager);
}


bool ErfFitObjectStreamer::inMap = IOManager::addToMap("SctData::ErfFitObject", auto_ptr<Streamer>(new ErfFitObjectStreamer()));

shared_ptr<Streamable> ErfFitObjectStreamer::read(IStream& in, const IOManager& manager) const throw(LogicError, IoError) {
    shared_ptr<Streamable> f (new ErfFitObject());
    read(in, *f, manager);
    return f;
}

void ErfFitObjectStreamer::read(IStream& in, Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {
    FitObjectStreamer::read(in, ob, manager);
}



bool GausFitObjectStreamer::inMap = IOManager::addToMap("SctData::GausFitObject", auto_ptr<Streamer>(new GausFitObjectStreamer()));

shared_ptr<Streamable> GausFitObjectStreamer::read(IStream& in, const IOManager& manager) const throw(LogicError, IoError) {
    shared_ptr<Streamable> f (new GausFitObject());
    read(in, *f, manager);
    return f;
}

void GausFitObjectStreamer::read(IStream& in, Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {
    FitObjectStreamer::read(in, ob, manager);
}



bool TopHatFitObjectStreamer::inMap = IOManager::addToMap("SctData::TopHatFitObject", auto_ptr<Streamer>(new TopHatFitObjectStreamer()));

shared_ptr<Streamable> TopHatFitObjectStreamer::read(IStream& in, const IOManager& manager) const throw(LogicError, IoError) {
    shared_ptr<Streamable> f (new TopHatFitObject());
    read(in, *f, manager);
    return f;
}

void TopHatFitObjectStreamer::read(IStream& in, Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {
    FitObjectStreamer::read(in, ob, manager);
}

}
}
