#ifndef SCTDATA_IO_FITOBJECTIS_H
#define SCTDATA_IO_FITOBJECTIS_H

#include "Sct/Streamer.h"
#include "../FitObjectIOHelper.h"
#include "Sct/LogicErrors.h"
#include "Sct/IoExceptions.h"

using namespace Sct;

namespace SctData {
namespace IO {
    
/**
This class is a Streamer for reading/writing FitObjects
*/	
class FitObjectStreamer : public virtual Streamer {
public:
    //Streamer overrides
    virtual void write(OStream& out, const Streamable& ob, const IOManager& manager) const throw(LogicError, IoError);
    virtual void read(IStream& in, Streamable& ob, const IOManager& manager) const throw(LogicError, IoError);   

    
protected:	
    FitObjectIOHelper helper;			///< The IOHelper
    FitObjectStreamer() throw() {}	///< Constructor is private
};

//Now ISStreamers for all the different FitObjects

class ErfcFitObjectStreamer : public FitObjectStreamer {
public:    
    ///@todo: Needed to get round gcc295 bug
    virtual void read(IStream& in, Streamable& ob, const IOManager& manager) const throw(LogicError, IoError);       
    virtual shared_ptr<Streamable> read(IStream& in, const IOManager& manager) const throw(LogicError, IoError);         
    
private:
    ErfcFitObjectStreamer() throw() {}
    static bool inMap;					///< The static dummy variable for the ISManager map
};

class ErfFitObjectStreamer : public FitObjectStreamer {
public:    
    ///@todo: Needed to get round gcc295 bug
    virtual void read(IStream& in, Streamable& ob, const IOManager& manager) const throw(LogicError, IoError);       
    virtual shared_ptr<Streamable> read(IStream& in, const IOManager& manager) const throw(LogicError, IoError);         
    
private:
    ErfFitObjectStreamer() throw() {}
    static bool inMap;					///< The static dummy variable for the ISManager map
};

class TopHatFitObjectStreamer : public FitObjectStreamer {
public:    
    ///@todo: Needed to get round gcc295 bug
    virtual void read(IStream& in, Streamable& ob, const IOManager& manager) const throw(LogicError, IoError);       
    virtual shared_ptr<Streamable> read(IStream& in, const IOManager& manager) const throw(LogicError, IoError);         

private:
    TopHatFitObjectStreamer() throw() {}
    static bool inMap;					///< The static dummy variable for the ISManager map
};

class GausFitObjectStreamer : public FitObjectStreamer {
public:    
    ///@todo: Needed to get round gcc295 bug
    virtual void read(IStream& in, Streamable& ob, const IOManager& manager) const throw(LogicError, IoError);       
    virtual shared_ptr<Streamable> read(IStream& in, const IOManager& manager) const throw(LogicError, IoError);         
    
private:
    GausFitObjectStreamer() throw() {}
    static bool inMap;					///< The static dummy variable for the ISManager map
};

}
}

#endif //SCTDATA_IO_FITOBJECTIS_H
