#include "FitScanResultStreamer.h"
#include "../FitScanResult.h"
#include "../FitObject.h"
#include "Sct/VersionNotSupportedException.h"

using namespace Sct;
using namespace boost;

namespace SctData {
namespace IO {

FitScanResultStreamer::FitScanResultStreamer() throw() {}

bool FitScanResultStreamer::inMap = IOManager::addToMap("SctData::FitScanResult", auto_ptr<Streamer>(new FitScanResultStreamer()));


shared_ptr<Streamable> FitScanResultStreamer::read(IStream& in, const IOManager& manager) const throw(LogicError, IoError) {
    shared_ptr<Streamable> ad (&helper.create());
    read(in, *ad, manager);
    return ad;
}

void FitScanResultStreamer::write(OStream& out, const Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {
    //Let superclass do its worker
    ScanResultStreamer::write(out, ob, manager);

    //Downcast should always work!
    const FitScanResult& fit = dynamic_cast<const FitScanResult&>(ob);
    out << (unsigned) 3; // version number; 
    
    unsigned int n = 0;
    string className;
    
    n = fit.getNLinkFits();
    out << n;
    ///@todo modify when bug is fixed in IS
    if (n>0) out << fit.getLinkFit(0).getClassName();
    for (unsigned i=0; i<n; i++) {
        FitObject& fo = fit.getLinkFit(i);
        manager.writeImpl(out, fo, false);
    }
    
    n = fit.getNChipFits();
    out << n;
    if (n>0) out << fit.getChipFit(0).getClassName();
    for (unsigned i=0; i<n; i++) {
        FitObject& fo = fit.getChipFit(i);
        manager.writeImpl(out, fo, false);
    }
    
    n = fit.getNChannelFits();
    out << n;
    if (n>0) out << fit.getChannelFit(0).getClassName();
    for (unsigned i=0; i<fit.getNChannelFits(); i++) {
        FitObject& fo = fit.getChannelFit(i);
        manager.writeImpl(out, fo, false);
    }
    
    manager.writeImpl(out, fit.getDefects(), false);
}


void FitScanResultStreamer::read(IStream& in, Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {
    ScanResultStreamer::read(in, ob, manager);

    FitScanResult& fit = dynamic_cast<FitScanResult&>(ob);
    
    unsigned size, version;
    string className;
    
    in >> version; 
    if (version!=3) throw VersionNotSupportedException("FitScanResult version error", __FILE__, __LINE__);
 
    in >> size;    
    fit.setNLinkFits(size);
    if (size > 0) in >> className;
    for (unsigned i=0; i<size ; i++) {
        shared_ptr<FitObject> fo (dynamic_pointer_cast<FitObject>(manager.readImpl(in, className)));
	fit.setLinkFit(i, fo);
    }
    
    in >> size;    
    fit.setNChipFits(size);
    if (size > 0) in >> className;
    for (unsigned i=0; i<size ; i++) {
        shared_ptr<FitObject> fo (dynamic_pointer_cast<FitObject>(manager.readImpl(in, className)));
	fit.setChipFit(i, fo);
    }
    
    in >> size;    
    fit.setNChannelFits(size);
    if (size > 0) in >> className;
    for (unsigned i=0; i<size ; i++) {
        shared_ptr<FitObject> fo (dynamic_pointer_cast<FitObject>(manager.readImpl(in, className)));
	fit.setChannelFit(i, fo);
    }
    manager.readImpl(in, fit.getDefects(), false);
}

}
}
