#include "ScanResultStreamer.h"
#include "../ScanResult.h"
#include "Sct/VersionNotSupportedException.h"
#include "Sct/LogicErrors.h"
#include "Sct/IoExceptions.h"

#include <string>

using namespace std;
using namespace Sct;

namespace SctData {
namespace IO {

ScanResultStreamer::ScanResultStreamer() throw() {}

void ScanResultStreamer::write(OStream& out, const Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {
    //Downcast should always work!
    const ScanResult& raw = dynamic_cast<const ScanResult&>(ob);

    //Write out the version
    out << (unsigned short)0;

    manager.writeImpl(out, raw.getHeader(), false);
    manager.writeImpl(out, raw.getPoints(), false);
    manager.writeImpl(out, raw.getConfiguration(), false);
}

void ScanResultStreamer::read(IStream& in, Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {
    //Downcast should always work!
    ScanResult& raw = dynamic_cast<ScanResult&>(ob);

    //cout << "ScanResultIS refreshing" << endl;
    //read in the version number and check it.
    unsigned short version = (unsigned)-1;
    in >> version;
    //cout << "Version: " << version << endl;
    if (version != 0)
        throw VersionNotSupportedException("We only support version 0 in ScanResultStreamer", __FILE__, __LINE__);

    //Now read everthing else in.
    helper.set(raw);
    manager.readImpl(in, helper.getHeader(), false);
    //cout << "Done header" << endl;
    manager.readImpl(in, helper.getPoints(), false);
    //cout << "Done ScanPoints" << endl;
    manager.readImpl(in, helper.getConfiguration(), false);
    //cout << "Done all "<<endl;
}
}
}
