#include "ModuleDefectList.h"
#include "ModuleDefect.h"
#include <iomanip>

namespace SctData {

ModuleDefectList::ModuleDefectList() throw() {}

//Delete all defects done by auto_ptr
ModuleDefectList::~ModuleDefectList() throw() {}
	
    void ModuleDefectList::addDefect(const ModuleDefect& defect, const ModuleElement& element) throw() {
	defectList.push_back(shared_ptr<const ModuleDefect>(new ModuleDefect(defect, element)));
    }
    
    void ModuleDefectList::add( shared_ptr<const ModuleDefect> defect) throw() {
        defectList.push_back(defect);
    }
    
    ModuleDefectList& ModuleDefectList::operator+=(const ModuleDefectList& toadd) throw() {
        for (unsigned i=0; i<toadd.defectList.size(); ++i) {
	    this->add(toadd.defectList[i]);
	}
	return *this;
    }

    ostream& ModuleDefectList::print(ostream& out) const throw() {
	for (unsigned int i=0; i<defectList.size(); ++i) {
	    out << setfill(' ') << setw(20) << defectList[i]->getName() 
		<<  "\tStart channel: " << defectList[i]->getModuleElement().getFirst() 
		<< "\t# of channels: " << defectList[i]->getModuleElement().getNChannels() 
		<< "\t" << defectList[i]->getDescription() << endl;
	}
	out << "[ total number of defects = " <<  defectList.size() << " ]" << endl;
	out << endl;
	return out;
    }

    const vector< shared_ptr<const ModuleDefect> >& ModuleDefectList::getAllDefects() const throw() {
    return defectList;
    }

    int ModuleDefectList::getNDefectsAffectingElement(const ModuleElement& el) const throw(Sct::LogicError) {
	int count = 0;
	for (unsigned i=0; i<defectList.size(); ++i) {
	    if (defectList[i]->getModuleElement().overlaps(el)) count++;
	}
	return count;
    }
    int ModuleDefectList::getNDefectsEncompassingElement(const ModuleElement& el) const throw(Sct::LogicError) {
	int count = 0;
	for (unsigned i=0; i<defectList.size(); ++i) {
	    if (defectList[i]->getModuleElement().contains(el)) count++;
	}
	return count;
    }

    bool ModuleDefectList::severeDefectAffectingElement(const ModuleElement& el) const throw(Sct::LogicError) {
	for (unsigned int i=0; i<defectList.size(); ++i) {
	    if (defectList[i]->getModuleElement().overlaps(el) && defectList[i]->isSevere()) return true;
	}
	return false;
    }

    bool ModuleDefectList::severeDefectEncompassingElement(const ModuleElement& el) const throw(Sct::LogicError) {
	for (unsigned int i=0; i<defectList.size(); ++i) {
	    if (defectList[i]->getModuleElement().contains(el) && defectList[i]->isSevere()) return true;
	}
	return false;
    }
    
    bool ModuleDefectList::defectAffectingElement(const ModuleElement& el, const ModuleDefect& defect) const throw(Sct::LogicError) {
	for (unsigned int i=0; i<defectList.size(); ++i) {
	    if ( defectList[i]->isOfTheSameTypeAs(defect) && 
		 defectList[i]->getModuleElement().overlaps(el) ) return true;
	}
	return false;
    }

     bool ModuleDefectList::defectEncompassingElement(const ModuleElement& el, const ModuleDefect& defect) const throw(Sct::LogicError) {
	for (unsigned int i=0; i<defectList.size(); ++i) {
	    if ( defectList[i]->isOfTheSameTypeAs(defect) && 
		 defectList[i]->getModuleElement().contains(el) ) return true;
	}
	return false;
    }
    
    bool ModuleDefectList::defectAffectingElement(const ModuleElement& el) const throw(Sct::LogicError) {
	for (unsigned int i=0; i<defectList.size(); ++i) {
	    if ( defectList[i]->getModuleElement().overlaps(el) ) return true;
	}
	return false;
    }

    bool ModuleDefectList::defectEncompassingElement(const ModuleElement& el) const throw(Sct::LogicError) {
	for (unsigned int i=0; i<defectList.size(); ++i) {
	    if ( defectList[i]->getModuleElement().contains(el) ) return true;
	}
	return false;
    }

    shared_ptr<ModuleDefectList> ModuleDefectList::getDefectsAffectingElement(const ModuleElement& el) const throw(Sct::LogicError) {
	shared_ptr<ModuleDefectList> newDefects(new ModuleDefectList());
	for (unsigned int i=0; i<defectList.size(); ++i) {
	    if (defectList[i]->getModuleElement().overlaps(el)) newDefects->add(defectList[i]);
	}
	return newDefects;
    }
    
    shared_ptr<ModuleDefectList> ModuleDefectList::getDefectsEncompassingElement(const ModuleElement& el) const throw(Sct::LogicError) {
	 shared_ptr<ModuleDefectList> newList(new ModuleDefectList()) ;
	 for (unsigned int i=0; i<defectList.size(); ++i) {
	     if (defectList[i]->getModuleElement().contains(el)) newList->add(defectList[i]);
	 }
	 return newList;
     }

    string ModuleDefectList::getClassName() const throw() {
	return "SctData::ModuleDefectList";
    }

    
}
