#ifndef RAWSCANRESULT_H
#define RAWSCANRESULT_H

#include "ScanResult.h"
#include <boost/shared_ptr.hpp>

class TH1;
class TH2;
class TH2D;
using Sct::LogicError;
using boost::shared_ptr;

namespace SctData {

class ModuleDefectList;

/**
 * A RawScanResult represents the raw data for a single scan.
 * Note that this object (like all others in the package) owns its members (i.e. it will delete them when it is deleted).
 * This can cause problems with the 4 argument constructor if you give it local vars!!!!
 * @author Matthew Palmer
 */
class RawScanResult : public ScanResult {
public:

    /// Creates a RawScanResult object with the given parameters
    RawScanResult(const ScanHeader& scanInfo, const ModuleConfiguration& module, const ScanPoints& points, TH2D& scan_link0, TH2D& scan_link1) throw() ;

    /// Destroys this object and all its members.
    virtual ~RawScanResult() throw();
    
    //Overrides	
    virtual string getClassName() const throw() ;	
    virtual string getUniqueID() const throw() ;
    
    /// Gets the unique ID this class would have if it existed and was constructed with header.
    static string getUniqueID(const ScanHeader& header) throw() ;

    /**
     * Gets the scan data as a ROOT histogram. 
     * @return the scan data
     * @param link should be 0 or 1
     */
    TH2D& getScanData(unsigned link) const throw(LogicError) ;

    /**
     * Gets the name a RawScanResult object would have given a Scan object.
     * Sets the pointer name to the name.  name is owned by the calling code (and must
     * therefore delete it).
     */
    //static void getsName(const Scan& scan, char*& name);

    /*
    //Projection functions are depreciated
    
    /// get a ROOT TH1 for the occupancy for link `ilink' with errors. ilink from {0,1}
    TH1*  getLinkOccupancy (const char* name, unsigned ilink, const ModuleDefectList* defects=0) const throw(LogicError) ;
    
    /// get a ROOT TH1 for the occupancy for chip `ichip' with errors. ichip from {0,11}
    TH1*  getChipOccupancy (const char* name, unsigned ichip, const ModuleDefectList* defects=0) const throw(LogicError) ;
    
    /// get a ROOT TH1 for the occupancy for chip `ichip' with errors. ilink from {0,1} ichip from {0,5}
    TH1*  getChipOccupancy (const char* name, unsigned ilink, unsigned ichip, const ModuleDefectList* defects=0) const throw(LogicError) ;
    
    /// get a ROOT TH1 for the occupancy for channel `ichan' with errors. ichan from {0,1535}
    TH1*  getChannelOccupancy (const char* name, unsigned istrip, const ModuleDefectList* defects=0) const throw(LogicError) ;

    /// get a ROOT TH1 for the occupancy for channel `ichan' with errors. ilink from {0,1}, ichan from {0,767}
    TH1*  getChannelOccupancy (const char* name, unsigned ilink, unsigned ichan, const ModuleDefectList* defects=0) const  throw(LogicError) ;
    */
protected:
    /**
     * Utility function that gets a general occupancy hist.
     * Returns a TH1D containing the occupancy.
     */
    //TH1* getProjection(const char* name, unsigned ilink, unsigned lowChan, unsigned highChan, const ModuleDefectList* defects=0) const throw(LogicError) ;
    
    shared_ptr<TH2D> scanData_link0; 			///<Scan data for link 0
    shared_ptr<TH2D> scanData_link1; 			///<Scan data for link 1

private:
    friend class RawScanResultIOHelper;
    RawScanResult() throw() ; 				///< Constructor for IO use only
};
}
#endif //#ifndef RAWSCANRESULT_H


