#include "SctData/NoiseOccupancyTestResult.h"
#include "SctData/ModuleDefectList.h"
#include "SctData/ModuleDefect.h"
#include "SctData/FitScanResult.h"
#include "SctData/RawScanResult.h"

#include "Sct/IS/IOManagerIS.h"
#include "Sct/SctNames.h"

#include "is/isinfo.h"

#include "TF1.h"
#include "TGraph.h"
#include "TApplication.h"
#include "TCanvas.h"

#include <iostream>
#include <string>
#include <iomanip>

using namespace std;
using namespace SctData;
using namespace Sct;
using namespace Sct::IS;
using namespace boost;


//Show a NoiseOccupancyTestResult - takes 1 arg the name in IS

int Help() {
    cout << "This program shows the contents of StrobeDelay TestResults.  It takes one argument - the name of the NPtGainTestResult in IS." << endl;
    return 0;
}

TCanvas* createCanvas(string name, string title) {
    return new TCanvas(name.c_str(), title.c_str());
}

int main(int argc, char** argv) {
    setExceptionHandlers(argv[0]);
    
    if (argc == 1) return Help();
    
    //Create TApplication - necessary to prevent batch mode and no graphics on later ROOT versions
    TApplication myapp("myapp", 0, 0);

    shared_ptr<Serializable> ob = IOManagerIS::instance().read(string(argv[1]));

    shared_ptr<NoiseOccupancyTestResult> result = dynamic_pointer_cast<NoiseOccupancyTestResult>(ob);
    if (!result) {
        cerr << "Errgh, didn't get a NoiseOccupancyTestResult - who knows why??? " << ob << endl;
        cout << ob->getClassName() << endl;
	return -1;
    }
    
    //Now print out defects:
    cout << endl << "Defects: " << endl;
    result->getDefects().print(cout);
    cout << endl;

    TCanvas* c = createCanvas(result->getModuleName() + "_NO", result->getModuleName() + " NO");
    c->Divide(4,3);
    for (unsigned int i=0; i<12; ++i) {
	ChipNOResult& r = result->getChipResult(i);
	cout << "Chip " << i << " noise: " << r.getNoise() << "  Fit params: ";
	for (int j=0; j<r.fit->GetNpar(); ++j) cout << r.fit->GetParameter(j) << "   ";
	cout << endl;
        c->cd(i+1);
	r.graph->Draw("ap");
	r.fit->SetLineWidth(1);
	r.fit->SetLineColor(2);
	r.fit->SetFillStyle(0);
	r.fit->Draw("lsame");	
    }


    myapp.Run();
}
