#include "SctData/ModuleConfiguration.h"
#include "SctData/ScanHeader.h"
#include "SctData/ConfigurationVariable.h"
#include "Sct/IS/IOManagerIS.h"
#include "Sct/SctNames.h"
#include "SctData/RawScanResult.h"
#include "SctData/OccupancyProjector.h"
#include "SctData/ModuleElement.h"

#include <is/isinfo.h>

#include <TH1.h>
#include <TH2.h>
#include <TCanvas.h>
#include <TApplication.h>

#include <iostream>
#include <vector>

using namespace Sct;
using namespace SctData;
using namespace Sct::IS;
using namespace boost;

/**
 * This is a simple test program that retrieves a RawScanResult object from an IS server.
 * The API is very likely to change and this may become obsolete quickly.
 */

static TApplication& getRootApp() {
    //Create TApplication - necessary to prevent batch mode and no graphics on later ROOT versions
    static TApplication myapp("myapp", 0, 0);
    return myapp;
}


void handleNewData(const RawScanResult& scan) {
    cout << "Scan: " << endl;
    const ModuleConfiguration& mc = scan.getConfiguration();
    const ConfigurationVariable& var = scan.getHeader().getVariable();
    cout << "Variable: " << var.getVariableName() << "  Strategy: " << var.getStrategyDescription() << endl;
    cout << scan.getUniqueID() << endl ;
    cout << scan.getHeader().getScanNumber() << endl << scan.getHeader().getRunNumber() << endl << scan.getHeader().getModuleName() << endl;
    const ScanPoints& s = scan.getPoints();
    cout << s.getPoint(0) << "  " << s.getNEvents(0) << "  " << s.getNPoints() << endl;
    //for (int i=0; i<s.getNPoints(); ++i) {
    //cout << s.getPoint(i) << "  " << s.getNEvents(i) << endl;
    //}
    string name = var.getVariableName() + scan.getUniqueID();
    TCanvas* c = new TCanvas(name.c_str(), name.c_str());
    c->Divide(1,2);
    c->cd(1);
    scan.getScanData(0).Draw("colz");
    cout << scan.getScanData(0).GetNbinsY() << endl;
    c->cd(2);
    scan.getScanData(1).Draw("colz");
    cout << "Done" << endl;

    //Now show some projections for channels and chips
    OccupancyProjector occ(scan);

    //Veto, but only for links and chips....
    //occ.vetoMaskedChannels(mc);

    name = "Projections" + scan.getUniqueID();
    c = new TCanvas(name.c_str(), name.c_str());
    c->Divide(3,2);
    c->cd(1);
    name = "Link0" + scan.getUniqueID();
    auto_ptr<TH1> link0 = occ.getOccupancy(name.c_str(), Link(0));
    link0->Draw();
    c->cd(2);
    name = "chip0" + scan.getUniqueID();
    auto_ptr<TH1> chip0 = occ.getOccupancy(name.c_str(), Chip(0));
    chip0->Draw();
    c->cd(3);
    
    occ.unsetVetos();
    
    name = "chan0" + scan.getUniqueID();
    auto_ptr<TH1> chan0 = occ.getOccupancy(name.c_str(), Channel(0));
    chan0->Draw();
    c->cd(4);
    name = "chan767" + scan.getUniqueID();
    auto_ptr<TH1> chan767 = occ.getOccupancy(name.c_str(), Channel(767));
    chan767->Draw();
    c->cd(5);
    name = "chan768" + scan.getUniqueID();
    auto_ptr<TH1> chan768 = occ.getOccupancy(name.c_str(), Channel(768));
    chan768->Draw();
    c->cd(6);
    name = "chan1535" + scan.getUniqueID();
    auto_ptr<TH1> chan1535 = occ.getOccupancy(name.c_str(), Channel(1535));
    chan1535->Draw();   
    
    //Start ROOT command loop
    getRootApp().Run();
}

int Help() {
    cout << "Retrieves Raw Data from IS.  Takes 1 argument - a name in IS for" << endl;
    return 0;
}

int main(int argc, char** argv) {
    setExceptionHandlers(argv[0]);
    
    //Create ROOT app
    getRootApp();
    
    if (argc == 1)
        return Help();

    cout << "Hello?" << endl;

    //Load it.
    shared_ptr<Serializable> ob = IOManagerIS::instance().read(string(argv[1]));
    shared_ptr<RawScanResult> scan = dynamic_pointer_cast<RawScanResult>(ob);

    cout << "Successfully retrieved Scan: " << argv[1] << endl;

    if (scan != 0) {
        handleNewData(*scan);
    } else {
        cout << "Error reading: " << argv[1] << endl;
    }
}
