#include "SctData/StrobeDelayTestResult.h"
#include "SctData/ModuleDefectList.h"
#include "SctData/ModuleDefect.h"
#include "Sct/SctNames.h"
#include "Sct/Serializable.h"
#include "SctData/FitScanResult.h"
#include "SctData/ConfigurationVariable.h"
#include "Sct/IS/IOManagerIS.h"
#include "Sct/IS/IONameIS.h"

#include <TCanvas.h>
#include <TSystem.h>
#include <is/isinfo.h>
#include <iomanip>
#include <fstream>
#include <string>
#include <memory>
#include <boost/scoped_ptr.hpp>

using namespace Sct;
using namespace Sct::IS;
using namespace SctData;
using namespace std;
using namespace boost;

// Test program looks for fitted strobe-delay tests, and plots their results.
// Takes 1 argument - the name of the StrobeDelayTestResult in IS


int Help() {
    cout << "This program shows the contents of StrobeDelay TestResults.  It takes one argument - the name of the NPtGainTestResult in IS." << endl;
    return 0;
}


int main(int argc, char** argv) {
    setExceptionHandlers(argv[0]);

    if (argc == 1) return Help();

    shared_ptr<Serializable> ob (IOManagerIS::instance().read(string(argv[1])));
    if (!ob) {
	cerr << "Couldn't get " << argv[1] << " from IS" << endl;
	return -3;
    }
    
    shared_ptr<StrobeDelayTestResult> result = dynamic_pointer_cast<StrobeDelayTestResult>(ob);
    
    if (!result) {
        cerr << "Errgh, didn't get a StrobeDelayTestResult - who knows why??? " << ob.get() << endl;
        cout << ob->getClassName() << endl;
        return -1;
    }
    
    if (result->getNScans()==0) {
        cout << "empty scan"<<endl;
        return -2;
    }

    cout <<result->getUniqueID()<<endl;

    unsigned int runNumber  = result->getRunNumber();
    string moduleName   = result->getModuleName();
    unsigned int scanNumber = result->getScanNumberAt(0);

    //Now print out defects:
    cout << endl << "Defects: " << endl;
    result->getDefects().print(cout) << endl;
    
    /*// output a file with these results.
    char filename[200];
    sprintf (filename,"StrobeDelay_%d_%d_%s.txt",runNumber,scanNumber,moduleName.c_str());
    ofstream file(filename);
    file << "UINT32 setmid = tapi.findModule(\"" << moduleName << "\")"<<endl;*/
    for (unsigned int ichip=0; ichip<result->getNOptima(); ++ichip) {
	cout << "StrobeDelay for chip: " << ichip << "  " << setprecision(2) << (int) (result->getOptimum(ichip)+0.5) << endl;
        //file << "tapi.modifyABCDVar(setmid,"<<ichip<<",3,"<< setprecision(2) << (int) (result->getOptimum(ichip)+0.5) <<")"<<endl;
    }
    //file.close();
    //cout <<"Wrote File "<<filename<<endl;
}
