#define USELIBRARY

#include <iostream>

#include "../TApi.h"

using namespace std;

int main (int argc, char **argv) {
  TApi tapi; //  = new TApi();

  tapi.initialiseAll(0);

  bool cont = true;

  int repetitions = 1;
  int length = 2;

  if(argc > 1) {
    for(int i=1; i<argc; i++) {
      if(argv[i][0] == '-') {
        switch(argv[i][1]) {
        case 'r':
          repetitions = atoi(&argv[i][2]);
          break;
        case 'l':
          length = atoi(&argv[i][2]);
          break;
        }
      }
    }
  }

  // Create and Send a simple ECHO primitive
  long *echoData; // [2] = {0xDEADF00D,0};

  echoData = new long[length];

  if(length < 3) {
    if(length > 0) echoData[0] = 0xDEADF00D;
    if(length > 1) echoData[1] = 0x0;
  } else {
    for (int i=0; i<length; i++) {
      echoData[i] = 0xBEAD0000 + i;
    }
  }

  try {
    for(int rep=0; rep<repetitions; rep++) {
      try {
        if(cont)
          tapi.echo(0, 0, 0, length, echoData);
      } catch(...) {
        cout << "Exception sending echo primitive\n";
        cont = false;
      }

      try {
        if(cont) { 
          //         cerr << "Going to await response\n";
          tapi.awaitResponse(0, 0, 0);
          //         cerr << "Done await response\n";
        }
      } catch(...) {
        cout << "Exception awaiting echo response\n";
        cont = false;
      }

      unsigned long outLength;

      unsigned long *outBody = tapi.getResponse(0, 0, 0, &outLength);
      if(outBody) {
        if(rep == 0) {
          // Print results (User processing of outList)
          unsigned long outIndex = (unsigned long)(outBody[1]);
          unsigned long outNumPrims = outBody[2];
          unsigned long outPrimVersion = outBody[3];
          cerr << "outLength = " << outLength << ", outIndex = " << outIndex
               << ", outNumPrims = " << outNumPrims << ", outPrimVersion = "
               << outPrimVersion << '\n';
          int outPtr = 4;
          for (unsigned int j=0; j<outNumPrims; j++) {
            unsigned long primLength = outBody[outPtr++];
            unsigned long primIndex = outBody[outPtr++];
            unsigned long primId = outBody[outPtr++];
            unsigned long primVersion = outBody[outPtr++];
            cout << "primLength = " << primLength << ", primIndex = " << primIndex 
                 << ", primId = " << primId << ", primVersion = " << primVersion << '\n';
            cout << "ECHO data: \n";
            hex(cout);
            for (unsigned int i=0; i<primLength-4; i++) {
              cout.width(8);
              cout << outBody[i+outPtr] <<" ";
              if (0 == (i+1)%8) cout << endl; 
            }
            if (0 != (primLength-4)%8) cout << endl;
            dec(cout);
          }
        }

        bool correct = true;
        for(int i=0; i<length; i++) {
          if(outBody[i+8] != echoData[i]) {
            correct = false;

            cout << "Check failed! " << rep << " + " << i << "\n";

            break;
          }
        }

        if(correct) cerr << ".";
        //         cout << "EchoTest completed successfully?\n";

        delete [] outBody;
      } else {
        cout << "Response was null\n";
      }
    }
    cout << "Complete\n";
  } catch(...) { // SctPixelRod::BaseException &b) {
    cout << "Got exception!" << endl;
  }

  delete [] echoData;

  tapi.shutdownAll();
  //  delete tapi;
}
