package SctApi.gui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import Sct_SctApi.ScanPackage.ScanOptions;

public class ScanView extends JFrame {
    Sct_SctApi.SctApi api;
    Sct_SctApi.Scan scan;

    JComponent information;
    JComponent control;

    ControlWidgets ctlWidgets;

    public ScanView(Sct_SctApi.SctApi a, Sct_SctApi.Scan scn) {
        super("Scan");

        api = a;
        scan = scn;
//         module = mod;
//         config = conf;

        JPanel buttons = new JPanel();

        JButton printScan = new JButton("Print Scan");
        printScan.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    scan.print();
                }
            });
        buttons.add(printScan);

        JButton execute = new JButton("Execute Scan");
        execute.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    api.doScan(scan);
                }
            });
        buttons.add(execute);

        JButton close = new JButton("Close");
        close.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    dispose();
                }
            });
        buttons.add(close);

        JButton destroy = new JButton("Destroy");
        destroy.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    scan.destroy();
                    dispose();
                }
            });
        buttons.add(destroy);

        getContentPane().add(buttons, BorderLayout.SOUTH);

        updateInfoPanel();
        createControlPanel();
    }

    void updateInfoPanel() {
        if(information != null)
            getContentPane().remove(information);

        information = Box.createVerticalBox();

        information.add(new JLabel("ScanVariable1: " + scan.getScanVariable1()));
        information.add(new JLabel("ScanVariable2: " + scan.getScanVariable2()));
        information.add(new JLabel("NTrigs: " + scan.getNTrigs()));

        String flagString = "";

        flagString += " format: " + scan.getOption(ScanOptions.format);
        flagString += " full: " + scan.getOption(ScanOptions.full);
        flagString += " bits32: " + scan.getOption(ScanOptions.bits32);
        flagString += " loopCalLine: " + scan.getOption(ScanOptions.loopCalLine);
        flagString += " distSlave: " + scan.getOption(ScanOptions.distSlave);
        flagString += " debug: " + scan.getOption(ScanOptions.debug);

        information.add(new JLabel(flagString));

        double [] realPoints = scan.getScanPoints1();

//         System.out.println("Points: " + realPoints);

        Object [] myPoints = new Object[realPoints.length];

        for(int i=0; i<realPoints.length; i++) {
            myPoints[i] = new String("" + realPoints[i]);
//             System.out.println(" " + realPoints[i] + " " + myPoints[i]);
        }

        JList pointList = new JList(myPoints);
        //        pointList.setLayoutOrientation(JList.HORIZONTAL_WRAP);

        information.add(new JScrollPane(pointList, 
                                        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, 
                                        JScrollPane.HORIZONTAL_SCROLLBAR_NEVER));

        getContentPane().add(information, BorderLayout.NORTH);

        information.invalidate();
        getContentPane().validate();
        //information.doLayout();
        //        information.update();
    }

    class ControlWidgets {
        JTextField type1;
        JTextField start1;
        JTextField stop1;
        JTextField step1;

        JTextField type2;
        JTextField start2;
        JTextField stop2;
        JTextField step2;

        JTextField ntrigs;
        JTextField format;
        JTextField full;
        JTextField bits32;
        JTextField loopCalLine;
        JTextField distSlave;
        JTextField debug;
    }

    void createControlPanel() {
        control = Box.createVerticalBox();

        ctlWidgets = new ControlWidgets();


        JComponent scan1 = Box.createHorizontalBox();

        scan1.add(new JLabel("Scan type1"));
        ctlWidgets.type1 = new JTextField("1");
        scan1.add(ctlWidgets.type1);

        scan1.add(new JLabel("Start scan 1 value"));
        ctlWidgets.start1 = new JTextField("0.0");
        scan1.add(ctlWidgets.start1);

        scan1.add(new JLabel("Stop scan 1 value"));
        ctlWidgets.stop1 = new JTextField("100.0");
        scan1.add(ctlWidgets.stop1);
        
        scan1.add(new JLabel("Step scan 1 value"));
        ctlWidgets.step1 = new JTextField("10.0");
        scan1.add(ctlWidgets.step1);

        JButton update1 = new JButton("Update scan 1");
        update1.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    short type = Short.parseShort(ctlWidgets.type1.getText());
                    double start = Double.parseDouble(ctlWidgets.start1.getText());
                    double stop = Double.parseDouble(ctlWidgets.stop1.getText());
                    double step = Double.parseDouble(ctlWidgets.step1.getText());

                    System.out.println("Configuring scan with " + type + " " + start + " " + stop + " " + step);
                    scan.configure(type, start, stop, step);

                    updateInfoPanel();
                }
            });
        scan1.add(update1);

        control.add(scan1);


        JComponent scan2 = Box.createHorizontalBox();

        scan2.add(new JLabel("Scan type2"));
        ctlWidgets.type2 = new JTextField("1");
        scan2.add(ctlWidgets.type2);

        scan2.add(new JLabel("Start scan 2 value"));
        ctlWidgets.start2 = new JTextField("0.0");
        scan2.add(ctlWidgets.start2);

        scan2.add(new JLabel("Stop scan 2 value"));
        ctlWidgets.stop2 = new JTextField("100.0");
        scan2.add(ctlWidgets.stop2);
        
        scan2.add(new JLabel("Step scan 2 value"));
        ctlWidgets.step2 = new JTextField("10.0");
        scan2.add(ctlWidgets.step2);

        JButton update2 = new JButton("Update scan 2");
        update2.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    short type = Short.parseShort(ctlWidgets.type2.getText());
                    double start = Double.parseDouble(ctlWidgets.start2.getText());
                    double stop = Double.parseDouble(ctlWidgets.stop2.getText());
                    double step = Double.parseDouble(ctlWidgets.step2.getText());

                    System.out.println("Configuring scan with " + type + " " + start + " " + stop + " " + step);
                    scan.configure2(type, start, stop, step);

                    updateInfoPanel();
                }
            });
        scan2.add(update2);

        control.add(scan2);


        JComponent options = Box.createHorizontalBox();

        options.add(new JLabel("No. Triggers"));
        ctlWidgets.ntrigs = new JTextField("100");
        options.add(ctlWidgets.ntrigs);

        options.add(new JLabel("Format"));
        ctlWidgets.format = new JTextField("0");
        options.add(ctlWidgets.format);

        options.add(new JLabel("Bits32"));
        ctlWidgets.bits32 = new JTextField("1");
        options.add(ctlWidgets.bits32);
        
        options.add(new JLabel("Loop over calLine"));
        ctlWidgets.loopCalLine = new JTextField("0");
        options.add(ctlWidgets.loopCalLine);

        options.add(new JLabel("Slave distribution"));
        ctlWidgets.distSlave = new JTextField("0");
        options.add(ctlWidgets.distSlave);

        JButton update = new JButton("Update");
        update.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    short ntrigs = Short.parseShort(ctlWidgets.ntrigs.getText());
                    scan.setNTrigs(ntrigs);
                    short format = Short.parseShort(ctlWidgets.format.getText());
                    scan.setOption(ScanOptions.format, format);
                    short bits32 = Short.parseShort(ctlWidgets.bits32.getText());
                    scan.setOption(ScanOptions.bits32, bits32);
                    short loopCalLine = Short.parseShort(ctlWidgets.loopCalLine.getText());
                    scan.setOption(ScanOptions.loopCalLine, loopCalLine);
                    short distSlave = Short.parseShort(ctlWidgets.distSlave.getText());
                    scan.setOption(ScanOptions.distSlave, distSlave);

                    updateInfoPanel();
                }
            });
        options.add(update);

        control.add(options);


        getContentPane().add(control, BorderLayout.CENTER);
    }
}
