#include <unistd.h>
#include <stdlib.h>
#include <ipc/object.h>
#include <ipc/server.h>

#include <mrs/message.h>
#include <is/isinfotmpl.h>

#include <algorithm>
#include <sstream>
#include <memory>

#include "Sct_SctApi.hh"
#include "SctApiServer.h"
#include "sctConf/configipc.h"

#include "Sct/ILUMarshalling.h"
#include "sctConf/marshalling.h"
using namespace Sct;

static std::auto_ptr<IPCServer> ipcServer; 

SctApiServer *SctApiServer::singleton = 0;

/*
  ilu_Cardinal is long unsigned int
 */

SctApiServer::SctApiServer(const char* oid, IPCServer* server) : 
  SctApi(boost::shared_ptr<SctConfiguration::Configuration>(new SctConfiguration::ConfigIPC)), IPCObject(oid, server), scans(0), 
  mrsStream(server->partition()), 
  isDict(server->partition())
{
  publish();

  ISInfoInt isPresent(1);

  SctApi::setMrsStream(&mrsStream);
  SctApi::setIsDictionary(&isDict);

  mrsStream << "SctApiServer" << MRS_INFORMATION << MRS_QUALIF("SctApi") << MRS_TEXT("Constructor called") << ENDM;

  if(isDict.contains("SCTAPIServer.present")) {
    std::cout << "*** Filename already in IS, will run anyway but there could be another service running?\n";
  } else {
    isDict.insert("SCTAPIServer.present", isPresent);
  }

  singleton = this;
}

SctApiServer::~SctApiServer() 
{
  mrsStream << "SctApiServer" << MRS_INFORMATION << MRS_QUALIF("SctApi") << MRS_TEXT("Destructor called") << ENDM;

  isDict.remove("SCTAPIServer.present");

  singleton = 0;

  withdraw();

  std::cout << "In known scans\n";

  std::cout << "Forcefully deleting " << scans.size() << " scans\n";

  for(std::list<Sct_SctApi_T_Scan *>::iterator i = scans.begin();
      i!= scans.end(); 
      i++) {
    delete *i;
  }

}

int main(int argc, char** argv) 
{
    ///Dirty hack - sleep to give config_server a chance to startup.  Should get
    ///config_server to use PMG synchronisation stuff
    sleep(2);
    
#warning "Needs lots of locks to work properly!"
  IPCCore::init(true, false); // Try reentrant for DDC IS callbacks
  IPCPartition p("SCT");
  ipcServer.reset(new IPCServer("Test_Server", p));
  try {
    std::auto_ptr<SctApiServer> h(new SctApiServer("SCTSctApiServer", ipcServer.get()));

    // Try publishing it
    if (!h->publish()) {
      std::cout << "Failed to publish SctApi" << std::endl;
      return 0;
    } 

    std::cout << "Serving...\n";

    ipcServer->run();
    std::cout << "Run over" << std::endl;
    h->withdraw();
//    delete ipcServer;
  } catch(SctConfiguration::ConfigurationException &c) {
    std::cout << "ConfigurationException: " << c.what() << std::endl;

    MRSStream mrsStream(p);
    mrsStream << "SctApiServer" << MRS_FATAL << MRS_QUALIF("SctApi") << MRS_TEXT("Configuration server not running!") << ENDM;
    
    exit(1);
  }
}

void SctApiServer::initialiseAll (Sct_SctApiStatus *_status, ilu_Cardinal runNumber)
{
  mrsStream << "SctApiServer" << MRS_INFORMATION << MRS_QUALIF("SctApi") << MRS_TEXT("initialiseAll called") << ENDM;

  worker().initialiseAll(runNumber);

  ISInfoInt present(2);

  isDict.update("SCTAPIServer.present", present);
}

void SctApiServer::shutdownAll (Sct_SctApiStatus *_status)
{
  mrsStream << "SctApiServer" << MRS_INFORMATION << MRS_QUALIF("SctApi") << MRS_TEXT("shutdownAll called") << ENDM;

  worker().shutdownAll();

  ISInfoInt present(1);

  isDict.update("SCTAPIServer.present", present);
}

void SctApiServer::status (Sct_SctApiStatus *_status)
{
  worker().status();
}

void SctApiServer::awaitResponse (Sct_SctApiStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod)
{
  worker().awaitResponse((unsigned int)partition, (unsigned int)crate, (unsigned int)rod, 5);
}

Sct_SctApi_T_DataBlock SctApiServer::getResponse (Sct_SctApiStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod, 
                                           ilu_Cardinal * length)
{
  unsigned long *body = worker().getResponse((unsigned int)partition, (unsigned int)crate, (unsigned int)rod, *length);

  return copyArrayToILU<_Sct_SctApi_T_DataBlock_sequence, unsigned long>(body, *length);
}

void SctApiServer::flashLED (Sct_SctApiStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod, ilu_ShortCardinal slaveNumber, ilu_Cardinal period, ilu_Cardinal flashes)
{
  worker().flashLED((unsigned int)partition, (unsigned int)crate, (unsigned int)rod, slaveNumber, period, flashes);
}

void SctApiServer::echo (Sct_SctApiStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod, ilu_Cardinal length, Sct_SctApi_T_DataBlock data)
{
  unsigned long ourLength = data->Length();

  unsigned long *array = new unsigned long[ourLength];
  unsigned long *body = copyILUToArray<Sct_SctApi_T_DataBlock, unsigned long>(data, array, ourLength);

  worker().echo((unsigned int)partition, (unsigned int)crate, (unsigned int)rod, (unsigned int)length, (long*)body);
}

void SctApiServer::createDebugPrimList (Sct_SctApiStatus *_status)
{
  worker().createDebugPrimList();
}

void SctApiServer::addDebugPrimList (Sct_SctApiStatus *_status, ilu_Cardinal length, ilu_Integer index, ilu_Integer id, ilu_Integer version, Sct_SctApi_T_DataBlock body)
{
  unsigned long ourLength = body->Length();
  unsigned long *array = new unsigned long[ourLength];
  copyILUToArray(body, array, ourLength);

  worker().addDebugPrimList(length, index, id, version, array);
}

void SctApiServer::sendDebugPrimList (Sct_SctApiStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod)
{
  worker().sendDebugPrimList((unsigned int)partition, (unsigned int)crate, (unsigned int)rod);
}

void SctApiServer::sendDebugSlavePrimList (Sct_SctApiStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod, ilu_Cardinal slave, ilu_Boolean await, ilu_Boolean response)
{
  worker().sendDebugSlavePrimList((unsigned int)partition, (unsigned int)crate, (unsigned int)rod, slave, await, response);
}

void SctApiServer::setRunNumber (Sct_SctApiStatus *_status, ilu_Cardinal newRun)
{
  worker().setRunNumber(newRun);
}

void SctApiServer::setScanNumber (Sct_SctApiStatus *_status, ilu_Cardinal newScan)
{
  worker().setScanNumber(newScan);
}

void SctApiServer::loadConfiguration (Sct_SctApiStatus *_status, ilu_T_CString filename)
{
  worker().loadConfiguration(filename);
}

void SctApiServer::storeModuleConfigurations (Sct_SctApiStatus *_status)
{
  worker().storeModuleConfigurations();
}

void SctApiServer::configureAllModules (Sct_SctApiStatus *_status)
{
  worker().sendABCDModule(0xffffffff, 0);
#warning "How to configure all?"
//    worker().configureAllModules();
}

sctConf_T_ABCDModule* SctApiServer::retrieveModule (Sct_SctApiStatus *_status, ilu_Cardinal mid)
{
  ABCDModule *body = worker().retrieveModule(mid);

  return copyABCDModuleToILU(*body);
}

ilu_ShortInteger SctApiServer::dspBlockDump (Sct_SctApiStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod, 
                                             ilu_Integer dspStart, ilu_Integer numWords, ilu_Integer dspNumber)
{
  return worker().dspBlockDump((unsigned int)partition, (unsigned int)crate, (unsigned int)rod,
                             dspStart, numWords, dspNumber);
}

Sct_SctApi_T_DataBlock SctApiServer::dspBlockRead (Sct_SctApiStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod, ilu_Integer dspStart, ilu_Integer numWords, ilu_Integer dspNumber, ilu_Integer * length)
{
  unsigned long *body = worker().dspBlockRead((unsigned int)partition, (unsigned int)crate, (unsigned int)rod,
                                            dspStart, numWords, dspNumber, *length);

  return copyArrayToILU<_Sct_SctApi_T_DataBlock_sequence, unsigned long>(body, *length);
}

ilu_ShortInteger SctApiServer::dspBlockWrite (Sct_SctApiStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod,
                                              Sct_SctApi_T_DataBlock buffer, ilu_Cardinal dspAddress, 
                                              ilu_Integer numWords, ilu_Integer dspNumber)
{
  unsigned long length = buffer->Length();
  unsigned long *array = new unsigned long[length];
  copyILUToArray<Sct_SctApi_T_DataBlock, unsigned long>(buffer, array, length);

  // assert (length == numWords)

  return worker().dspBlockWrite((unsigned int)partition, (unsigned int)crate, (unsigned int)rod, 
                                array, dspAddress, numWords, dspNumber);
}

ilu_Cardinal SctApiServer::dspSingleRead (Sct_SctApiStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod, 
                                          ilu_Cardinal dspAddr, ilu_Integer dspNumber)
{
  return worker().dspSingleRead((unsigned int)partition, (unsigned int)crate, (unsigned int)rod, 
                              dspAddr, dspNumber);
}

void SctApiServer::dspSingleWrite (Sct_SctApiStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod, ilu_Cardinal dspAddr, ilu_Cardinal val, ilu_Integer dspNumber)
{
  worker().dspSingleWrite((unsigned int)partition, (unsigned int)crate, (unsigned int)rod, dspAddr, val, dspNumber);
}

Sct_SctApi_T_DataBlock SctApiServer::readSlaveDsp (Sct_SctApiStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod, ilu_ShortInteger s, ilu_Cardinal add, ilu_Cardinal words)
{
  unsigned long *body = worker().readSlaveDsp((unsigned int)partition, (unsigned int)crate, (unsigned int)rod, s, add, words);

  return copyArrayToILU<_Sct_SctApi_T_DataBlock_sequence, unsigned long>(body, words);
}

ilu_Cardinal SctApiServer::findModuleSerial (Sct_SctApiStatus *_status, ilu_T_CString sn)
{
  return worker().findModule(sn);
}

ilu_T_CString SctApiServer::convertToString (Sct_SctApiStatus *_status, ilu_Cardinal mid)
{
  std::string serialNum;
  
  serialNum = worker().convertToString(mid);

  ilu_T_CString sn = new char[serialNum.length()+1];

  memcpy(sn, serialNum.c_str(), serialNum.length());
  sn[serialNum.length()] = 0;

  return sn;
}

ilu_Cardinal SctApiServer::findModule (Sct_SctApiStatus *_status, ilu_ShortCardinal mur, ilu_ShortCardinal number)
{
  return worker().findModule(mur, number);
}

ilu_Cardinal SctApiServer::findBarrelModule (Sct_SctApiStatus *_status, 
                                             ilu_Cardinal barrel, ilu_Cardinal row, ilu_ShortInteger number)
{
  return worker().findBarrelModule(barrel, row, number);
}

void SctApiServer::convertToBarrelModule (Sct_SctApiStatus *_status, ilu_Cardinal mid, ilu_Cardinal * barrel, ilu_Cardinal * row, ilu_ShortInteger * number)
{
  int rnumber;
  worker().convertToBarrelModule((unsigned int)mid, *(unsigned int*)barrel, *(unsigned int*)row, rnumber);

  *number = rnumber;
}

ilu_Cardinal SctApiServer::findEndcapModule (Sct_SctApiStatus *_status, ilu_Integer disk, ilu_Integer ring, ilu_Integer number)
{
  return worker().findEndcapModule(disk, ring, number);
}

void SctApiServer::convertToEndcapModule (Sct_SctApiStatus *_status, ilu_Cardinal mid, ilu_Cardinal * disk, ilu_Cardinal * ring, ilu_Cardinal * number)
{
  worker().convertToEndcapModule((unsigned int)mid, *(unsigned int*)disk, *(unsigned int*)ring, *(unsigned int*)number);
}

void SctApiServer::getABCDModule (Sct_SctApiStatus *_status, ilu_Cardinal mid, ilu_Cardinal bank)
{
  worker().getABCDModule(mid, bank);
}

void SctApiServer::getABCDModules (Sct_SctApiStatus *_status, ilu_Cardinal bank)
{
  //cout << "SctApiServer::getABCDModules called " << bank << endl;
  worker().getABCDModule(bank);
  //cout << "SctApiServer::getABCDModules done" << endl;
}

void SctApiServer::setABCDModule (Sct_SctApiStatus *_status, ilu_Cardinal mid, ilu_Cardinal bank)
{
  worker().setABCDModule(mid, bank);
}

void SctApiServer::setABCDModules (Sct_SctApiStatus *_status, ilu_Cardinal bank)
{
  worker().setABCDModule(0xffffffff, bank);
}

void SctApiServer::sendABCDModule (Sct_SctApiStatus *_status, ilu_Cardinal mid, ilu_Cardinal bank, ilu_Cardinal type)
{
  worker().sendABCDModule(mid, bank, type);
}

void SctApiServer::sendABCDModules (Sct_SctApiStatus *_status, ilu_Cardinal bank)
{
  worker().sendABCDModule(0xffffffff, bank);
}

void SctApiServer::modifyABCDMask (Sct_SctApiStatus *_status, ilu_Cardinal mid, Sct_SctApi_T_DataBlock mask)
{
#warning "This needs checking!"
  UINT32 array[48];
  worker().modifyABCDMask(mid, copyILUToArray<Sct_SctApi_T_DataBlock, UINT32> (mask, array, 48));
}

void SctApiServer::modifyABCDTrims (Sct_SctApiStatus *_status, ilu_Cardinal mid, Sct_SctApi_T_CharBlock trims){
#warning "This needs checking!"
  UINT8 array[12*128];
  worker().modifyABCDTrims (mid, copyILUToArray<Sct_SctApi_T_CharBlock, UINT8> (trims, array, 12*128) );
} 

void SctApiServer::modifyABCDRC (Sct_SctApiStatus *_status, ilu_Cardinal mid, ilu_Cardinal chip, ilu_ShortCardinal index, ilu_Real p0, ilu_Real p1, ilu_Real p2)
{
  cout << "SctApiServer::modifyABCDRC called" << endl;
  worker().modifyABCDRC(mid, chip, index, p0, p1, p2);
}

void SctApiServer::modifyABCDVar (Sct_SctApiStatus *_status, ilu_Cardinal typ, ilu_Real var)
{
  worker().modifyABCDVar(typ, var);
}

void SctApiServer::modifyABCDModuleVar (Sct_SctApiStatus *_status, ilu_Cardinal mid, ilu_Cardinal typ, ilu_Real var)
{
  worker().modifyABCDVar(mid, typ, var);
}

void SctApiServer::modifyABCDChipVar (Sct_SctApiStatus *_status, ilu_Cardinal mid, ilu_Cardinal chip, ilu_Cardinal typ, ilu_Real var)
{
  worker().modifyABCDVar(mid, chip, typ, var);
}

void SctApiServer::modifyBOCParam (Sct_SctApiStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod, 
                                   ilu_Cardinal channel, ilu_Cardinal type, ilu_Cardinal val)
{
  worker().modifyBOCParam(partition, crate, rod, channel, type, val);
}

void SctApiServer::lasersOff (Sct_SctApiStatus *_status)
{
  worker().lasersOff();
}

void SctApiServer::freeTriggers (Sct_SctApiStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Real freq)
{
  worker().freeTriggers(partition, crate, freq);
}

void SctApiServer::stopTriggers (Sct_SctApiStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate)
{
  worker().stopTriggers(partition, crate);
}

void SctApiServer::timL1A (Sct_SctApiStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate)
{
  worker().timL1A(partition, crate);
}

void SctApiServer::timCalL1A (Sct_SctApiStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_ShortInteger delay)
{
  worker().timCalL1A(partition, crate, delay);
}

void SctApiServer::sendTriggerAll (Sct_SctApiStatus *_status, class Sct_SctApi_T_Trigger * trig)
{
  TriggerServer *realTrigger = dynamic_cast<TriggerServer*>(trig);

  IPCObjectVar<TriggerServer>::duplicate(realTrigger);
  worker().sendTriggerAll(realTrigger);
  realTrigger->decRef();
}

void SctApiServer::sendTrigger (Sct_SctApiStatus *_status, 
                                ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod, class Sct_SctApi_T_Trigger * trig)
{
  TriggerServer *realTrigger = dynamic_cast<TriggerServer*>(trig);

  IPCObjectVar<TriggerServer>::duplicate(realTrigger);
  worker().sendTrigger(partition, crate, rod, realTrigger);
  realTrigger->decRef();
}

void SctApiServer::doScan (Sct_SctApiStatus *_status, class Sct_SctApi_T_Scan * thisScan)
{
  mrsStream << "SctApiServer" << MRS_INFORMATION << MRS_QUALIF("SctApi") << MRS_TEXT("Starting scan") << ENDM;

  ISInfoInt scanning(1);
  isDict.insert("SCTAPIServer.scanning", scanning);

  ScanServer *realScan = dynamic_cast<ScanServer*>(thisScan);

  IPCObjectVar<ScanServer>::duplicate(realScan);
  worker().doScan(*realScan);
  realScan->decRef();

  isDict.remove("SCTAPIServer.scanning");

  mrsStream << "SctApiServer" << MRS_INFORMATION << MRS_QUALIF("SctApi") << MRS_TEXT("Scan has completed") << ENDM;
}

void SctApiServer::doRawScan (Sct_SctApiStatus *_status, class Sct_SctApi_T_Scan * thisScan, ilu_Integer delay)
{
  mrsStream << "SctApiServer" << MRS_INFORMATION << MRS_QUALIF("SctApi") << MRS_TEXT("Raw scan started") << ENDM;

  ISInfoInt scanning(1);
  isDict.insert("SCTAPIServer.scanning", scanning);

  ScanServer *realScan = dynamic_cast<ScanServer*>(thisScan);

  IPCObjectVar<ScanServer>::duplicate(realScan);
  worker().doRawScan(*realScan, delay);
  realScan->decRef();

  isDict.remove("SCTAPIServer.scanning");

  mrsStream << "SctApiServer" << MRS_INFORMATION << MRS_QUALIF("SctApi") << MRS_TEXT("Raw scan completed") << ENDM;
}

void SctApiServer::printBOCSetup (Sct_SctApiStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod)
{
  worker().printBOCSetup(partition, crate, rod);
}

void SctApiServer::printABCDModule (Sct_SctApiStatus *_status, ilu_Cardinal mid)
{
  worker().printABCDModule(mid);
}

void SctApiServer::printABCDRodModule (Sct_SctApiStatus *_status, ilu_Cardinal mid, ilu_ShortInteger bank)
{
  worker().printABCDRodModule(mid, bank);
}

void SctApiServer::decodeEvent (Sct_SctApiStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod, ilu_ShortInteger sl, ilu_ShortInteger index, ilu_Boolean extFlag, ilu_Boolean errorType)
{
  worker().decodeEvent(partition, crate, rod, sl, index, extFlag, errorType);
}

void SctApiServer::rawData (Sct_SctApiStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod, ilu_Integer delay, ilu_Integer units, ilu_Boolean setMask)
{
  worker().rawData(partition, crate, rod, delay, units, setMask);
}

void SctApiServer::rawDataWithTrigger (Sct_SctApiStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod, ilu_Integer delay, ilu_Integer units, ilu_Boolean setMask, class Sct_SctApi_T_Trigger * trig)
{
  TriggerServer *realTrigger = dynamic_cast<TriggerServer*>(trig);

  IPCObjectVar<TriggerServer>::duplicate(realTrigger);
  worker().rawData(partition, crate, rod, delay, units, setMask, realTrigger);
  realTrigger->decRef();
}

void SctApiServer::sendL1A (Sct_SctApiStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod)
{
  worker().sendL1A(partition, crate, rod);
}

Sct_SctApi_T_CharBlock SctApiServer::probe (Sct_SctApiStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod)
{
  return copyVectorToILU<_Sct_SctApi_T_CharBlock_sequence, char>(worker().probe(partition, crate, rod));
}

Sct_SctApi_T_CharBlock SctApiServer::probeHarness (Sct_SctApiStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod, ilu_Cardinal harness)
{
  return copyVectorToILU<_Sct_SctApi_T_CharBlock_sequence, char>(worker().probe(partition, crate, rod, harness));
}

Sct_SctApi_T_CharBlock SctApiServer::probeWithTrigger (Sct_SctApiStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod, class Sct_SctApi_T_Trigger * trig)
{
  TriggerServer *realTrigger = dynamic_cast<TriggerServer*>(trig);

  IPCObjectVar<TriggerServer>::duplicate(realTrigger);
  vector<char> ret = worker().probeWithTrigger(partition, crate, rod, realTrigger);
  realTrigger->decRef();
  return copyVectorToILU<_Sct_SctApi_T_CharBlock_sequence, char>(ret);
}

Sct_SctApi_T_CharBlock SctApiServer::probeHarnessWithTrigger (Sct_SctApiStatus *_status, ilu_Cardinal partition, ilu_Cardinal crate, ilu_Cardinal rod, class Sct_SctApi_T_Trigger * trig, ilu_Cardinal harness)
{
  TriggerServer *realTrigger = dynamic_cast<TriggerServer*>(trig);

  IPCObjectVar<TriggerServer>::duplicate(realTrigger);
  vector<char> ret = worker().probeWithTrigger(partition, crate, rod, realTrigger, harness);
  realTrigger->decRef();
  return copyVectorToILU<_Sct_SctApi_T_CharBlock_sequence, char>(ret);
}

void SctApiServer::requestHardReset (Sct_SctApiStatus *_status, ilu_Cardinal mid)
{
  worker().requestHardReset(mid);
}

void SctApiServer::requestIVCurve (Sct_SctApiStatus *_status, ilu_Cardinal mid, ilu_ShortReal start, ilu_ShortReal stop, ilu_ShortReal step, ilu_ShortCardinal delay, ilu_ShortReal currentLim) 
{
  worker().requestIVCurve(mid, start, stop, step, delay, currentLim);
}

void SctApiServer::setSelect (Sct_SctApiStatus *_status, ilu_Cardinal mid, ilu_Boolean value)
{
  worker().setSelect(mid, value);
}

void SctApiServer::changeRunState (Sct_SctApiStatus *_status, ilu_Integer state)
{
  worker().changeRunState(state);
}

void SctApiServer::ddcGetChannelParameter (Sct_SctApiStatus *_status, ilu_Integer crate, ilu_Integer channel, ilu_T_CString dpeName, ilu_Integer * dpeValue, ilu_Integer timeout)
{
  int value;
  worker().ddcGetChannelParameter(crate, channel, string(dpeName), value, timeout);
  *dpeValue = value;
}

Sct_SctApi_T_Scan * SctApiServer::defaultScan (Sct_SctApiStatus *_status)
{
  std::cout << "Asked for default scan\n";

  ScanServer *defScan = new ScanServer(ipcServer.get());

  scans.push_back(defScan);

  return IPCObjectVar<ScanServer>::duplicate(defScan);
}

Sct_SctApi_T_Trigger * SctApiServer::defaultTrigger (Sct_SctApiStatus *_status)
{
  std::cout << "Asked for default trigger\n";

  TriggerServer *defTrig = new TriggerServer(ipcServer.get());

  return IPCObjectVar<TriggerServer>::duplicate(defTrig);
}

Sct_SctApi_T_ScanList SctApiServer::knownScans (Sct_SctApiStatus *_status)
{
  std::cout << "In known scans\n";

  Sct_SctApi_T_ScanList myScans;

  myScans = new _Sct_SctApi_T_ScanList_sequence(10);

  std::cout << "Copying sequence of length " << scans.size() << std::endl;

  for(std::list<Sct_SctApi_T_Scan *>::iterator i=scans.begin();
      i != scans.end();
      i++) {
    ScanServer *thisScan = IPCObjectVar<ScanServer>::duplicate(dynamic_cast<ScanServer *>(*i));
    myScans->Append(thisScan);
  }

  return myScans;
}

SctApi::SctApi &SctApiServer::worker() 
{
  return *this;
}

/**************** ScanServer ****************/

ScanServer::ScanServer(IPCServer* server) : SctApi::ScanDefImpl(), IPCObject(getServerName().c_str(), server)
{
  std::cout << "Create a new ScanServer\n";

//    dynamic_cast<SctApi::Scan*>(this)->print();
  std::cout << "trigsPerBurst = " << worker().getNTrigs() << std::endl; 

  TriggerServer *temp = new TriggerServer(ipcServer.get());

  std::cout << "TrigServer: " << temp;
  ScanDefImpl::setTrigger1(IPCObjectVar<TriggerServer>::duplicate(temp));
  std::cout << " getTrigger1() " << worker().getTrigger1() << std::endl;
  ScanDefImpl::setTrigger2(IPCObjectVar<TriggerServer>::duplicate(new TriggerServer(ipcServer.get())));

//    if(!publish()) {
//      cerr << "Failed to publish new scan" << std::endl;
//    }

//    std::cout << "Published scan\n";
}

ScanServer::~ScanServer() 
{
//    return IPCObjectVar<ScanServer>::release(this);
  
//    withdraw();
}

void ScanServer::destroy(ipcStatus *_status) 
{
  std::list<Sct_SctApi_T_Scan *> &scans = SctApiServer::singleton->scans;

  std::list<Sct_SctApi_T_Scan *>::iterator item = find(scans.begin(), scans.end(), this);
  if(item == scans.end()) {
    std::cout << "*******Couldn't find destroyed scan in scans list**********!\n";
  } else {
    scans.erase(item);
  }
  decRef();
}


int ScanServer::nextNumber = 0;

std::string ScanServer::getServerName() {
  std::ostringstream serverName;
  serverName << "ScanServer" << nextNumber++;
  return serverName.str();
}

void ScanServer::print(Sct_SctApiStatus *_status) 
{
  std::cout << "Try to print a scan\n";
  worker().print();
  std::cout << "Gave up or sucess (printing scan)\n";
}

void ScanServer::configure (Sct_SctApiStatus *_status, ilu_ShortCardinal type, ilu_Real start, ilu_Real stop, ilu_Real step) 
{
  worker().configure(type, start, stop, step);
}

void ScanServer::configure2 (Sct_SctApiStatus *_status, ilu_ShortCardinal type, ilu_Real start, ilu_Real stop, ilu_Real step) 
{
  worker().configure2(type, start, stop, step);
}

class Sct_SctApi_T_Trigger * ScanServer::getTrigger1 (Sct_SctApiStatus *_status) 
{
// //   return IPCObjectVar<TriggerServer>::duplicate(worker().getTrigger1());

// //   TriggerServer *trig = dynamic_cast<TriggerServer*>(worker().getTrigger1());

// //   if(!trig) {
// #warning "Returning made up one"

  TriggerServer *defTrig = dynamic_cast<TriggerServer *> (worker().getTrigger1()); // new TriggerServer(ipcServer);

  return IPCObjectVar<TriggerServer>::duplicate(defTrig);
}

class Sct_SctApi_T_Trigger * ScanServer::getTrigger2 (Sct_SctApiStatus *_status) 
{
// #warning "Returning made up one"
  TriggerServer *defTrig = dynamic_cast<TriggerServer *> (worker().getTrigger2()); // new TriggerServer(ipcServer);

  return IPCObjectVar<TriggerServer>::duplicate(defTrig);
//   TriggerServer *defTrig = new TriggerServer(ipcServer);

//   return IPCObjectVar<TriggerServer>::duplicate(defTrig);
}

void ScanServer::setTrigger1 (Sct_SctApiStatus *_status, class Sct_SctApi_T_Trigger * trig)
{
#warning "Unimplemented"
}

void ScanServer::setTrigger2 (Sct_SctApiStatus *_status, class Sct_SctApi_T_Trigger * trig) 
{
#warning "Unimplemented"
}

void ScanServer::setScanVariable1 (Sct_SctApiStatus *_status, ilu_ShortCardinal var) 
{
  worker().setScanVariable1(var);
}

void ScanServer::setScanVariable2 (Sct_SctApiStatus *_status, ilu_ShortCardinal var) 
{
  worker().setScanVariable2(var);
}

ilu_ShortCardinal ScanServer::getScanVariable1 (Sct_SctApiStatus *_status) 
{
  return worker().getScanVariable1();
}

ilu_ShortCardinal ScanServer::getScanVariable2 (Sct_SctApiStatus *_status) 
{
  return worker().getScanVariable2();
}

void ScanServer::setNTrigs (Sct_SctApiStatus *_status, ilu_Cardinal nTrigs) 
{
  worker().setNTrigs(nTrigs);
}

ilu_ShortCardinal ScanServer::getNTrigs (Sct_SctApiStatus *_status) 
{
  return worker().getNTrigs();
}

Sct_SctApi_T_ScanPoints ScanServer::getScanPoints1 (Sct_SctApiStatus *_status) 
{
  Sct_SctApi_T_ScanPoints result;

  result = _Sct_SctApi_T_ScanPoints_sequence::Create(4, NULL);
  result->Clear(0);

  for(unsigned long i=0; i<worker().getScanPoints1().size(); i++) {
    result->Append(worker().getScanPoints1()[i]);
  }

  return result;
}

Sct_SctApi_T_ScanPoints ScanServer::getScanPoints2 (Sct_SctApiStatus *_status)
{
  Sct_SctApi_T_ScanPoints result;

  result = _Sct_SctApi_T_ScanPoints_sequence::Create(4, NULL);
  result->Clear(0);

  for(unsigned long i=0; i<worker().getScanPoints2().size(); i++) {
    result->Append(worker().getScanPoints2()[i]);
  }

  return result;
}

static void enumerateToList(ilu_Real val, void *list) {
  SctApi::Scan::ScanPoints &myList = *(SctApi::Scan::ScanPoints *)list;

  myList.push_back(val);
}

void ScanServer::setScanPoints1 (Sct_SctApiStatus *_status, Sct_SctApi_T_ScanPoints scans) 
{
  SctApi::Scan::ScanPoints newPoints;

  scans->Enumerate(enumerateToList, &newPoints);

  worker().setScanPoints1(newPoints);
}
 
void ScanServer::setScanPoints2 (Sct_SctApiStatus *_status, Sct_SctApi_T_ScanPoints scans) 
{
  SctApi::Scan::ScanPoints newPoints;

  scans->Enumerate(enumerateToList, &newPoints);

  worker().setScanPoints2(newPoints);
}

ilu_Integer ScanServer::getOption (Sct_SctApiStatus *_status, Sct_SctApi_T_Scan_ScanOptions opt) 
{
  return worker().getOption((ScanOptions)opt);
}

void ScanServer::setOption (Sct_SctApiStatus *_status, Sct_SctApi_T_Scan_ScanOptions opt, ilu_Integer val) 
{
  worker().setOption((ScanOptions)opt, val);
}

ilu_Cardinal ScanServer::getScanNumber (Sct_SctApiStatus *_status) 
{
  return worker().getScanNumber();
}
 
void ScanServer::setScanNumber (Sct_SctApiStatus *_status, ilu_Cardinal scan) 
{
  worker().setScanNumber(scan);
}

ilu_Cardinal ScanServer::getRunNumber (Sct_SctApiStatus *_status) 
{
  return worker().getRunNumber();
}

void ScanServer::setRunNumber (Sct_SctApiStatus *_status, ilu_Cardinal run)
{
  worker().setRunNumber(run);
}

sctConf_T_ModuleList ScanServer::getModuleList (Sct_SctApiStatus *_status, ilu_Cardinal group) 
{
  sctConf_T_ModuleList result;

  result = _sctConf_T_ModuleList_sequence::Create(4, NULL);
  result->Clear(0);

  std::list<std::string> myList = worker().getModuleList(group);
  for(std::list<std::string> :: const_iterator m = myList.begin();
      m!=myList.end();
      m++) {
    char *copy = (char *)malloc((m->length()+1) * sizeof(char));
    strncpy(copy, m->c_str(), m->length());
    copy[m->length()] = 0;
    result->Append(copy);
  }

  return result;
}

ilu_Cardinal ScanServer::getNGroups (Sct_SctApiStatus *_status) 
{
  return worker().getNGroups();
}

SctApi::Scan &ScanServer::worker() 
{
  return *this;
}


/******************* TriggerServer **********************/

TriggerServer::TriggerServer(IPCServer* server) 
  : SctApi::TriggerImpl(), IPCObject(getServerName().c_str(), server)
{
}

TriggerServer::~TriggerServer()
{
}

void TriggerServer::destroy(ipcStatus *_status) 
{
  decRef();
}

int TriggerServer::nextNumber = 0;

std::string TriggerServer::getServerName() {
  std::ostringstream serverName;
  serverName << "TriggerServer" << nextNumber++;
  std::cout << "Trigger name: " << serverName.str() << std::endl;;
  return serverName.str();
}

void TriggerServer::print (Sct_SctApiStatus *_status) 
{
  std::cout << "Try to print a trigger\n";
  worker().print();
  std::cout << "Succeeded in printing a trigger\n";
}

void TriggerServer::singleL1A (Sct_SctApiStatus *_status) 
{
  worker().singleL1A();
}

void TriggerServer::doubleL1A (Sct_SctApiStatus *_status, ilu_ShortCardinal delay) 
{
  worker().doubleL1A(delay);
}

void TriggerServer::calL1A (Sct_SctApiStatus *_status, ilu_ShortCardinal delay) 
{
  worker().calL1A(delay);
}

void TriggerServer::pulseL1A (Sct_SctApiStatus *_status, ilu_ShortCardinal delay) 
{
  worker().pulseL1A(delay);
}

void TriggerServer::softL1A (Sct_SctApiStatus *_status, ilu_ShortCardinal delay) 
{
  worker().softL1A(delay);
}

void TriggerServer::softCalL1A (Sct_SctApiStatus *_status, ilu_ShortCardinal delay, ilu_ShortCardinal delay2) 
{
  worker().softCalL1A(delay, delay2);
}

void TriggerServer::softPulseL1A (Sct_SctApiStatus *_status, ilu_ShortCardinal delay, ilu_ShortCardinal delay2) 
{
  worker().softPulseL1A(delay, delay2);
}

void TriggerServer::bcL1A (Sct_SctApiStatus *_status, ilu_ShortCardinal delay) 
{
  worker().bcL1A(delay);
}

void TriggerServer::bcCalL1A (Sct_SctApiStatus *_status, ilu_ShortCardinal delay, ilu_ShortCardinal delay2) 
{
  worker().bcCalL1A(delay, delay2);
}

void TriggerServer::bcPulseL1A (Sct_SctApiStatus *_status, ilu_ShortCardinal delay, ilu_ShortCardinal delay2) 
{
  worker().bcPulseL1A(delay, delay2);
}

void TriggerServer::setCommIncr (Sct_SctApiStatus *_status, ilu_ShortCardinal command, ilu_ShortCardinal incr)
{
  worker().setCommIncr(command, incr);
}

void TriggerServer::getCommIncr (Sct_SctApiStatus *_status, ilu_ShortCardinal * command, ilu_ShortCardinal * incr)
{
  worker().getCommIncr(*command, *incr);
}

SctApi::Trigger &TriggerServer::worker() 
{
  return *this;
}
