/*
 * ScriptingInterface.java
 *
 * Created on 28 November 2002, 14:34
 */

package TestGUI;
import org.apache.bsf.*;
import GuiComponents.scripting.*;
import GuiComponents.Console.*;
import javax.swing.*;

/**
 *
 * @author  palmer
 */
public class ScriptingInterface extends javax.swing.JPanel {
    protected BSFManager manager;
    protected JConsole console;     //Only allow one!
    protected SearchableObjectRegistry registry;
    
    /** Creates new form ScriptingInterface */
    public ScriptingInterface(MyJPanel parent) {
        manager = new BSFManager();
        initComponents();
        initMyComponents();
        initManager(parent);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;
        
        jButtonPanel = new javax.swing.JPanel();
        bJavaScript = new javax.swing.JButton();
        bBeanShell = new javax.swing.JButton();
        bJython = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        
        setLayout(new java.awt.BorderLayout());
        
        jButtonPanel.setLayout(new java.awt.GridBagLayout());
        
        bJavaScript.setText("JavaScript");
        bJavaScript.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bJavaScriptActionPerformed(evt);
            }
        });
        
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        jButtonPanel.add(bJavaScript, gridBagConstraints);
        
        bBeanShell.setText("BeanShell");
        bBeanShell.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bBeanShellActionPerformed(evt);
            }
        });
        
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        jButtonPanel.add(bBeanShell, gridBagConstraints);
        
        bJython.setText("Jython");
        bJython.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bJythonActionPerformed(evt);
            }
        });
        
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        jButtonPanel.add(bJython, gridBagConstraints);
        
        jLabel1.setText("Create new scripting console...");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        jButtonPanel.add(jLabel1, gridBagConstraints);
        
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jButtonPanel.add(jLabel2, gridBagConstraints);
        
        add(jButtonPanel, java.awt.BorderLayout.CENTER);
        
    }//GEN-END:initComponents

    private void bJythonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bJythonActionPerformed
        createScriptInterface(new JythonFactory());
    }//GEN-LAST:event_bJythonActionPerformed

    private void initMyComponents() {
        
    }
    
    private void initManager(MyJPanel parent) {
        registry = new SearchableObjectRegistry();
        manager.setObjectRegistry(registry);
        
        try {
            manager.declareBean("sctPanel", parent, parent.getClass());
            manager.declareBean("sct", parent.si, parent.si.getClass());
        } catch (BSFException be) {
            System.err.println("Error initializing manager: " + be);
        }
    }
    
    private void bBeanShellActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bBeanShellActionPerformed
        createScriptInterface(new BeanShellFactory());
    }//GEN-LAST:event_bBeanShellActionPerformed

    private void bJavaScriptActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bJavaScriptActionPerformed
        createScriptInterface(new JavaScriptFactory());
    }//GEN-LAST:event_bJavaScriptActionPerformed
    
    /**
     * Creates a window into which users can type/load scripts.
     * @param sf The ScriptFactory to use to create the necessary objects.
     */
    protected void createScriptInterface(ScriptFactory sf) {
        JConsole console = sf.createConsole();
        try {
            manager.declareBean("console",  console,  console.getClass());
        } catch (BSFException bsfe) {
            System.err.println("Internal error, BSFException whilst adding bean console.\n" + bsfe);
            bsfe.printStackTrace();
        }
        BSFEngine engine = sf.createEngine(manager);
        NameCompleter nc = sf.createNameCompleter(engine, registry);
        DefaultInterpreter di = sf.createInterpreter(engine, console, nc);        
        addNewConsole(console, di.toString());
    }
    
    /**
     * Adds the new console to this window...
     */
    protected boolean addNewConsole(JConsole console, String suffix) {    
        if (this.console != null) {
            MyJPanel.getMessageArea().append("Can only have 1 scripting window open at once\n");
            return false;
        }
        this.console = console;
        JFrame frame = new JFrame();
        frame.addWindowListener(new java.awt.event.WindowAdapter() {            
            public void windowClosing(java.awt.event.WindowEvent evt) {
                removeConsole();
            }
        });
        frame.getContentPane().add(console);  
        frame.setTitle("Scripting Window:" + suffix);
        frame.show();
        frame.setSize(400, 500);        
        return true;
    }
    
    /** 
     * Removes this JConsole
     */
    protected void removeConsole() {        
        try {
            manager.undeclareBean("console");
        } catch (BSFException bsfe) {
            System.err.println("Internal error, BSFException whilst removing bean console.\n" + bsfe);
            bsfe.printStackTrace();
        }
        console = null;
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton bBeanShell;
    private javax.swing.JButton bJavaScript;
    private javax.swing.JButton bJython;
    private javax.swing.JPanel jButtonPanel;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    // End of variables declaration//GEN-END:variables
    
    
}
