#include "ArchivingWorkerGroup.h"
#include "archiving/ArchRawScanResult.h"
#include "archiving/ArchFitScanResult.h"
#include "archiving/ArchNPtGainTestResult.h"
#include "archiving/ArchNoiseOccupancyTestResult.h"
#include "archiving/ArchStrobeDelayTestResult.h"
#include "archiving/ArchTrimRangeTestResult.h"
#include "archiving/ArchTimeWalkTestResult.h"
#include "archiving/ArchPipelineTestResult.h"
#include "archiving/ArchFullBypassTestResult.h"
#include "archiving/ArchNMaskTestResult.h"


#include "Sct/LogicErrors.h"

using namespace Sct;
using namespace SctData;
using namespace boost;

namespace SctArchivingService {
    
void ArchivingWorkerGroup::work(shared_ptr<Serializable> ob) throw() {
    try {
	if (!ob) throw InvalidArgumentError("Failed to get object from IS", __FILE__, __LINE__);
	
	shared_ptr<NoiseOccupancyTestResult> NOResult = dynamic_pointer_cast<NoiseOccupancyTestResult>(ob); 
	if(NOResult) {      
	    cout<<" The Test Obj is NoiseOccupancyTestResult "<<endl;      
	    ArchNoiseOccupancyTestResult archnotest(*NOResult);
	    archnotest.SetUNIXDir();
	    archnotest.SaveNoiseOccupancy();
	    cout<< "Noise Occupancy Test Correctly Saved " << endl;
	    return;
	}
	
	shared_ptr<NPtGainTestResult> NPtResult = dynamic_pointer_cast<NPtGainTestResult>(ob);
	if(NPtResult) {
	    cout<<" The Test Obj is NPtGainTestResult "<<endl;      
	    ArchNPtGainTestResult archNPtGtest(*NPtResult);
	    archNPtGtest.SetUNIXDir();
	    archNPtGtest.SaveNPtGain();
	    cout<< "NPt Gain Test Correctly Saved " << endl;
	    return;
	}
	
	shared_ptr<StrobeDelayTestResult> SDResult = dynamic_pointer_cast<StrobeDelayTestResult>(ob);
	if(SDResult) {
	    cout<<" The Test Obj is StrobeDelayTestResult "<<endl;      
	    ArchStrobeDelayTestResult archSDtest(*SDResult);
	    archSDtest.SetUNIXDir();
	    archSDtest.SaveStrobeDelay();
	    cout<< "Strobe Delay Test Correctly Saved " << endl;
	    return;
	}
	
	shared_ptr<TrimRangeTestResult> TRResult = dynamic_pointer_cast<TrimRangeTestResult>(ob);
	if(TRResult) {
	    cout<<" The Test Obj is TrimRangeTestResult "<<endl;      
	    ArchTrimRangeTestResult archTRtest(*TRResult);
	    archTRtest.SetUNIXDir();
	    archTRtest.SaveTrimRange();
	    cout<< "Trim Range Test Correctly Saved " << endl;
	    return;
	}

	shared_ptr<TimeWalkTestResult> TWResult = dynamic_pointer_cast<TimeWalkTestResult>(ob);
	if(TWResult) {
	    cout<<" The Test Obj is TimeWalkTestResult "<<endl;      
	    ArchTimeWalkTestResult archTWtest(*TWResult);
	    archTWtest.SetUNIXDir();
	    archTWtest.SaveTimeWalk();
	    cout<< "TimeWalk Test Correctly Saved " << endl;
	    return;
	}

	shared_ptr<FullBypassTestResult> FBResult = dynamic_pointer_cast<FullBypassTestResult>(ob);
	if(FBResult) {
	    cout<<" The Test Obj is FullBypassTestResult "<<endl;      
	    ArchFullBypassTestResult archFBtest(*FBResult);
	    archFBtest.SetUNIXDir();
	    archFBtest.SaveFullBypass();
	    cout<< "FullBypass Test Correctly Saved " << endl;
	    return;
	}


	shared_ptr<PipelineTestResult> PResult = dynamic_pointer_cast<PipelineTestResult>(ob);
	if(PResult) {
	    cout<<" The Test Obj is PipelineTestResult "<<endl;      
	    ArchPipelineTestResult archPtest(*PResult);
	    archPtest.SetUNIXDir();
	    archPtest.SavePipeline();
	    cout<< "Pipeline Test Correctly Saved " << endl;
	    return;
	}
	

	shared_ptr<NMaskTestResult> NMResult = dynamic_pointer_cast<NMaskTestResult>(ob);
	if(NMResult) {
	    cout<<" The Test Obj is NMaskTestResult "<<endl;      
	    ArchNMaskTestResult archNMtest(*NMResult);
	    archNMtest.SetUNIXDir();
	    archNMtest.SaveNMask();
	    cout<< "NMask Test Correctly Saved " << endl;
	    return;
	}
	
	throw InvalidArgumentError("The object on IS is an unknown Test type: " + ob->getClassName(), __FILE__, __LINE__);
    } catch(Throwable& e) {
	e.sendToMrs(MRS_ERROR);
    }
}    

}
