#include <stdlib.h>
#include <ipc/object.h>
#include <ipc/server.h>

#include "sctConf.hh"
#include "configArchiveServer.h"
#include "configserver.h"

using namespace SctConfiguration;

typedef sctConf_T_ilu__prefix_idlExceptionType_ConfigurationException ConfException;

static void myRaise(sctConfStatus *_status, const ConfigurationException &c);

using namespace std;

IPCServer* ipcServer;

ConfigArchiveServer::ConfigArchiveServer(const char* oid, IPCServer* server) : IPCObject(oid, server) 
{
}

ConfigArchiveServer::~ConfigArchiveServer() 
{
}

sctConf_T_Configuration * ConfigArchiveServer::getOldRunConfiguration (sctConfStatus *_status, ilu_Cardinal runNumber)
{
  ConfigServer *oldConfig = new ConfigServer("oldarchiveFilename", ipcServer);

  configs.push_back(oldConfig);

  myRaise(_status, ConfigurationException("Archive not found!"));

  return IPCObjectVar<ConfigServer>::duplicate(oldConfig);
}

int main(int argc, char** argv) 
{
  IPCCore::init(false);   // No multithreads
  IPCPartition p("SCT");
  ipcServer = new IPCServer(sctConf_C_ConfigurationArchive_instanceName, p);

  ConfigArchiveServer *h = new ConfigArchiveServer(sctConf_C_ConfigurationArchive_serverName, ipcServer);
  if (!h->publish()) {
    cout << "Failed to publish Configuration Archive" << endl;
    return 0;
  } 
  cout << "Configuration Archiving server published, awaiting commands\n";
  try {
    ipcServer->run();
  } catch(ConfigurationException &c) {
    cout << "Configuration run threw exception " << c.what() << endl;
  }

  cout << "Configuration Archive Server run over" << endl;

  h->withdraw();
  delete h;
  delete ipcServer;
}

void myRaise(sctConfStatus *_status, const ConfigurationException &c) 
{
  ConfException *e = new ConfException;
  char *buffer = new char[strlen(c.what())+1];

  cout << "Raising exception " << c.what() << endl;

  strcpy(buffer, c.what());
  buffer[strlen(c.what())] = 0;
  e->detail = buffer;
  sctConf_G::RaiseException(_status, sctConf_E_ConfigurationException, e);
}
