package DisplayGUI;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import java.awt.Graphics2D.*;
import java.awt.geom.*;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;

class ColorScalePane extends JPanel implements MouseListener, MouseMotionListener, displayParams {
    gui guiControl;
 
        FontMetrics metrics;

	int maxWidth,maxHeight;
        Dimension dim;
        Rectangle area;
        Rectangle2D.Double drawRect;
        GradientPaint colorGradient;
        boolean firstTime=true;
        int x,y;
        
        ColorScale colorScale;
        String scaleDescription;


        public ColorScalePane(gui parent){
          guiControl=parent;
          setBackground(backgroundColor);
          addMouseMotionListener(this);
          addMouseListener(this);
          dim = getSize();
          maxWidth = (int)dim.getWidth();

          setMinimumSize(new Dimension(300,60));
          setPreferredSize(new Dimension(300,60));
          setBorder(new javax.swing.border.TitledBorder(null, "Display Colour Scale", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Arial", 0, 11), new java.awt.Color(0, 0, 153)));
    }


    public void setColorScale() {
// multiple switches (yuk) to set color scale
        top:
        switch(guiControl.displayStatus.getStatus()) {
        case DisplayStatus.IS_MONITOR:
           int noScans= guiControl.isInterface.getNoScans();  // get the number of scans in this test
           if(noScans==0) {
            colorScale=null;
            scaleDescription=null;
            }
           else {
            if(guiControl.selectorPane.getDisplayParameter().indexOf("Analysed")!=-1) noScans=1; // only 1 test result object
            if(noScans==1) {
               colorScale = new ColorScale(0.0, java.awt.Color.red, "0");
               colorScale.add(1.0, java.awt.Color.green, "1");
               }
            else {
               colorScale = new ColorScale(0.0, java.awt.Color.red, "0",(double)noScans,java.awt.Color.green,Integer.toString(noScans));
               scaleDescription = new String("Number of IS data objects");
               }
            }
          break;
        case DisplayStatus.MODULE_GROUP_SELECTION:
          colorScale = new ColorScale(-1.0,java.awt.Color.white, "None");
          colorScale.add(0.0, java.awt.Color.ORANGE, "Group 1");
          colorScale.add(1.0, java.awt.Color.cyan,"Group 2"); 
          scaleDescription = new String("Selected module grouping");
          break top;
        case DisplayStatus.TEST_DATA:
          switch(guiControl.displayPane.getDisplayOption()) {
            case 0:
            case 1:
              setTestDataScales(guiControl.testDataInterface.getTestIndex());
              break;
            default:
              colorScale = new ColorScale(-1.0,java.awt.Color.white, "No Data");
              colorScale.add(1.0, java.awt.Color.green,"Good");
              colorScale.add(2.0, java.awt.Color.red,"Poor"); 
              scaleDescription = new String("Comparison with SCTDB Reference");    
              }         
          break top;
        case DisplayStatus.SCTDB_DATA:
          setTestDataScales(guiControl.sctdbDataInterface.getTestIndex());
          break top;
        default:
          colorScale=null;
          scaleDescription=null;
          break;
          }
        repaint();
        } 

    public void setTestDataScales(int testIndex) {
          int parameterIndex = guiControl.selectorPane.getDisplayParameterIndex();
          top2:
          switch(testIndex) {
             case TestDataInfo.TEST_NMASK:
               switch(parameterIndex) {
                     case 0:  // test status
                       colorScale = new ColorScale(0.0, java.awt.Color.green, "Pass");
                       colorScale.add(1.0,java.awt.Color.orange, "Problem");
                       colorScale.add(2.0,java.awt.Color.red, "Fail");
                       scaleDescription = new String("Test Status");
                       break top2;
                     case 1:  //ndefects
                       colorScale = new ColorScale(0.0, java.awt.Color.green, "0",15.0,java.awt.Color.red,"15");
                       scaleDescription = new String("Number of defective channels");
                       break top2;
                      default:
                       colorScale=null;
                       scaleDescription=null;
                       break top2;
                     }
             case TestDataInfo.TEST_FULLBYPASS:
               switch(parameterIndex) {
                     case 0:  // test status
                       colorScale = new ColorScale(0.0, java.awt.Color.green, "Pass");
                       colorScale.add(1.0,java.awt.Color.orange, "Problem");
                       colorScale.add(2.0,java.awt.Color.red, "Fail");
                       scaleDescription = new String("Test Status");
                       break top2;
                     case 1:  //ndefects
                       colorScale = new ColorScale(0.0, java.awt.Color.green, "0",15.0,java.awt.Color.red,"15");
                       scaleDescription = new String("Number of defective channels");
                       break top2;
                     case 2:  //Token
                       colorScale = new ColorScale(0.0, java.awt.Color.orange, "0",5.0,java.awt.Color.green,"5.0");
                       scaleDescription = new String("Token");
                       break top2;
                     case 3:  //RToken
                       colorScale = new ColorScale(0.0, java.awt.Color.orange, "0",5.0,java.awt.Color.green,"5.0");
                       scaleDescription = new String("R-Token");
                       break top2;
                      default:
                       colorScale=null;
                       scaleDescription=null;
                       break top2;
                     }
             case TestDataInfo.TEST_PIPELINE:
               switch(parameterIndex) {
                     case 0:  // test status
                       colorScale = new ColorScale(0.0, java.awt.Color.green, "Pass");
                       colorScale.add(1.0,java.awt.Color.orange, "Problem");
                       colorScale.add(2.0,java.awt.Color.red, "Fail");
                       scaleDescription = new String("Test Status");
                       break top2;
                     case 1:  //ndefects
                       colorScale = new ColorScale(0.0, java.awt.Color.green, "0",15.0,java.awt.Color.red,"15");
                       scaleDescription = new String("Number of defective channels");
                       break top2;
                     case 2:
                       colorScale = new ColorScale(0., java.awt.Color.green, "0");
                       colorScale.add(1.0,java.awt.Color.red,">0");
                       scaleDescription = new String("Number of defective pipelines");
                       break top2;
                      default:
                       colorScale=null;
                       scaleDescription=null;
                       break top2;
                     }
             case TestDataInfo.TEST_STROBEDELAY:
               switch(parameterIndex) {
                     case 0:  // test status
                       colorScale = new ColorScale(0.0, java.awt.Color.green, "Pass");
                       colorScale.add(1.0,java.awt.Color.orange, "Problem");
                       colorScale.add(2.0,java.awt.Color.red, "Fail");
                       scaleDescription = new String("Test Status");
                       break top2;
                     case 1:  //ndefects
                       colorScale = new ColorScale(0.0, java.awt.Color.green, "0",15.0,java.awt.Color.red,"15");
                       scaleDescription = new String("Number of defective channels");
                       break top2;
                     case 2:
                       colorScale = new ColorScale(0.0, java.awt.Color.red, "0",20.0,java.awt.Color.red,"20");
                       colorScale.add(10.0,java.awt.Color.green,"10");
                       colorScale.add(15.0,java.awt.Color.green,"15");
                       scaleDescription = new String("Strobe Delay");
                       break top2;
                      default:
                       colorScale=null;
                       scaleDescription=null;
                       break top2;
                      }
               
             case TestDataInfo.TEST_NPTGAIN:
             case TestDataInfo.TEST_3PTGAIN:
                switch(parameterIndex) {
                     case 0:  // test status
                       colorScale = new ColorScale(0.0, java.awt.Color.green, "Pass");
                       colorScale.add(1.0,java.awt.Color.orange, "Problem");
                       colorScale.add(2.0,java.awt.Color.red, "Fail");
                       scaleDescription = new String("Test Status");
                       break top2;
                     case 1:  //ndefects
                       colorScale = new ColorScale(0.0, java.awt.Color.green, "0",15.0,java.awt.Color.red,"15");
                       scaleDescription = new String("Number of defective channels");
                       break top2;
                     case 2:  //vt50
                       colorScale = new ColorScale(0.0, java.awt.Color.red, "0",300.0,java.awt.Color.red,"300.");
                       colorScale.add(100.0,java.awt.Color.green,"100.");
                       colorScale.add(200.0,java.awt.Color.green,"200.");
                       scaleDescription = new String("VT50 point at 2fC");
                       break top2;
                      case 3:  //vt50 rms
                       colorScale = new ColorScale(0.0, java.awt.Color.blue, "0",10.0,java.awt.Color.red,"10.");
                       colorScale.add(1.0,java.awt.Color.green,"1.");
                       colorScale.add(5.0,java.awt.Color.green,"5.");
                       scaleDescription = new String("VT50 rms at 2fC");
                       break top2;  
                      case 4:  //Gain
                       colorScale = new ColorScale(30.0, java.awt.Color.red, "30",70.0,java.awt.Color.red,"70");
                       colorScale.add(40.0,java.awt.Color.green,"40");
                       colorScale.add(60.0,java.awt.Color.green,"60");
                       scaleDescription = new String("Gain at 2fC");
                       break top2;        
                     case 5:  //Gain rms
                       colorScale = new ColorScale(0.0, java.awt.Color.blue, "0",10.0,java.awt.Color.red,"10.");
                       colorScale.add(1.0,java.awt.Color.green,"1.");
                       colorScale.add(5.0,java.awt.Color.green,"5.");
                       scaleDescription = new String("Gain rms at 2fC");
                       break top2;       
                      case 6:  //Offset
                       colorScale = new ColorScale(0.0, java.awt.Color.red, "0.",100.0,java.awt.Color.red,"100.");
                       colorScale.add(10.0,java.awt.Color.green,"10");
                       colorScale.add(90.0,java.awt.Color.green,"90");
                       scaleDescription = new String("Offset at 2fC");
                       break top2;      
                      case 7:  //Offset rms
                       colorScale = new ColorScale(0.0, java.awt.Color.blue, "0",10.0,java.awt.Color.red,"10.");
                       colorScale.add(1.0,java.awt.Color.green,"1.");
                       colorScale.add(5.0,java.awt.Color.green,"5.");
                       scaleDescription = new String("Offset rms at 2fC");
                       break top2;           
                      case 8:  //Output noise
                       colorScale = new ColorScale(10.0, java.awt.Color.blue, "10.",16.0,java.awt.Color.red,"16.");
                       colorScale.add(12.0,java.awt.Color.green,"12.");
                       colorScale.add(14.0,java.awt.Color.green,"14.");
                       scaleDescription = new String("Output Noise at 2fC");
                       break top2;  
                      case 9:  //Input noise 
                       colorScale = new ColorScale(1000.0, java.awt.Color.blue, "1000",2000.0,java.awt.Color.red,"2000");
                       colorScale.add(1300.0,java.awt.Color.green,"1300");
                       colorScale.add(1600.0,java.awt.Color.green,"1600");
                       scaleDescription = new String("Input noise (ENC) at 2fC");
                       break top2;        
                      case 10:  //Input noise rms
                       colorScale = new ColorScale(0.0, java.awt.Color.blue, "0.",100.0,java.awt.Color.red,"100");
                       colorScale.add(30.0,java.awt.Color.green,"30.");
                       colorScale.add(80.0,java.awt.Color.green,"80.");
                       scaleDescription = new String("Input noise rms (ENC) at 2fC");
                       break top2;   
                      default:
                       colorScale=null;
                       scaleDescription=null;
                       break top2;
                      }
            case TestDataInfo.TEST_NOISE:
                switch(parameterIndex) {
                     case 0:  // test status
                       colorScale = new ColorScale(0.0, java.awt.Color.green, "Pass");
                       colorScale.add(1.0,java.awt.Color.orange, "Problem");
                       colorScale.add(2.0,java.awt.Color.red, "Fail");
                       scaleDescription = new String("Test Status");
                       break top2;
                     case 1:  //ndefects
                       colorScale = new ColorScale(0.0, java.awt.Color.green, "0",15.0,java.awt.Color.red,"15");
                       scaleDescription = new String("Number of defective channels");
                       break top2;
                      case 2:  //Offset
                       colorScale = new ColorScale(0.0, java.awt.Color.red, "0.",100.0,java.awt.Color.red,"100.");
                       colorScale.add(10.0,java.awt.Color.green,"10");
                       colorScale.add(90.0,java.awt.Color.green,"90");
                       scaleDescription = new String("Offset");
                       break top2;   
                      case 3: // occupancy
                       colorScale = new ColorScale(0.0000001, java.awt.Color.blue, "1E-7",0.0001,java.awt.Color.red,"1E-4");
                       colorScale.add(0.000001,java.awt.Color.green,"1E-6");
                       colorScale.add(0.00001,java.awt.Color.green,"1E-5");
                       colorScale.setLogScale();
                       scaleDescription = new String("Noise Occupancy");  
                       break top2;
                      case 4: // occupancy rms
                       colorScale = new ColorScale(0.0000001, java.awt.Color.blue, "1E-7",0.0001,java.awt.Color.red,"1E-4");
                       colorScale.add(0.000001,java.awt.Color.green,"1E-6");
                       colorScale.add(0.00001,java.awt.Color.green,"1E-5");
                       colorScale.setLogScale();
                       scaleDescription = new String("Noise Occupancy RMS");  
                       break top2;    
                      case 5: // noise (enc)
                       colorScale = new ColorScale(1000.0, java.awt.Color.blue, "1000",2000.0,java.awt.Color.red,"2000");
                       colorScale.add(1300.0,java.awt.Color.green,"1300");
                       colorScale.add(1600.0,java.awt.Color.green,"1600");
                       scaleDescription = new String("Noise (ENC)");
                       break top2;        
                      default:
                       colorScale=null;
                       scaleDescription=null;
                       break top2;
                      }                      
            case TestDataInfo.TEST_TRIM:
                switch(parameterIndex) {
                     case 0:  // test status
                       colorScale = new ColorScale(0.0, java.awt.Color.green, "Pass");
                       colorScale.add(1.0,java.awt.Color.orange, "Problem");
                       colorScale.add(2.0,java.awt.Color.red, "Fail");
                       scaleDescription = new String("Test Status");
                       break top2;
                     case 1:  //ndefects
                       colorScale = new ColorScale(0.0, java.awt.Color.green, "0",15.0,java.awt.Color.red,"15");
                       scaleDescription = new String("Number of defective channels");
                       break top2;
                      case 2: // Range
                       colorScale = new ColorScale(0, java.awt.Color.blue, "0");
                       colorScale.add(1,java.awt.Color.green,"1");
                       colorScale.add(2,java.awt.Color.red,"2");
                       colorScale.add(3,java.awt.Color.yellow,"3");
                       scaleDescription = new String("Trim Range");
                       break top2;
                      case 3: // Target
                       colorScale = new ColorScale(0.0, java.awt.Color.blue, "0.",250.0,java.awt.Color.red,"250");
                       colorScale.add(50.0,java.awt.Color.green,"50.");
                       colorScale.add(200.0,java.awt.Color.green,"200.");
                       scaleDescription = new String("Target");  
                       break top2;
                      case 4: // Ntrim
                       colorScale = new ColorScale(0., java.awt.Color.green, "100%");
                       colorScale.add(1.0,java.awt.Color.red,"<100%");
                       scaleDescription = new String("Number of trimmed channels");
                       break top2;
                      case 5: // vt50
                       colorScale = new ColorScale(0.0, java.awt.Color.red, "0",300.0,java.awt.Color.red,"300.");
                       colorScale.add(100.0,java.awt.Color.green,"100.");
                       colorScale.add(200.0,java.awt.Color.green,"200.");
                       scaleDescription = new String("VT50 point");
                       break top2;
                      case 6:  //vt50 rms
                       colorScale = new ColorScale(0.0, java.awt.Color.blue, "0",10.0,java.awt.Color.red,"10.");
                       colorScale.add(1.0,java.awt.Color.green,"1.");
                       colorScale.add(5.0,java.awt.Color.green,"5.");
                       scaleDescription = new String("VT50 rms");
                       break top2;  
                      case 7:  //tr_offset
                       colorScale = new ColorScale(-20.0, java.awt.Color.blue, "0",0.0,java.awt.Color.green,"0.");
                       scaleDescription = new String("Trim offset");
                       break top2;  
                      case 8:  //tr_offset rms
                       colorScale = new ColorScale(0.0, java.awt.Color.blue, "0",2.0,java.awt.Color.green,"2.");
                       scaleDescription = new String("Trim offset rms");
                       break top2;  
                      case 9:  //tr_step
                       colorScale = new ColorScale(0.0, java.awt.Color.blue, "0",20.0,java.awt.Color.green,"20.");
                       scaleDescription = new String("Trim step size");
                       break top2; 
                      case 10:  //tr_step rms
                       colorScale = new ColorScale(0.0, java.awt.Color.blue, "0",1.0,java.awt.Color.green,"1.");
                       scaleDescription = new String("Trim step size rms");
                       break top2;  
                      default:
                       colorScale=null;
                       scaleDescription=null;
                       break top2;
                      }   
             case TestDataInfo.TEST_TIMEWALK:
               switch(parameterIndex) {
                     case 0:  // test status
                       colorScale = new ColorScale(0.0, java.awt.Color.green, "Pass");
                       colorScale.add(1.0,java.awt.Color.orange, "Problem");
                       colorScale.add(2.0,java.awt.Color.red, "Fail");
                       scaleDescription = new String("Test Status");
                       break top2;
                     case 1:  //ndefects
                       colorScale = new ColorScale(0.0, java.awt.Color.green, "0",15.0,java.awt.Color.red,"15");
                       scaleDescription = new String("Number of defective channels");
                       break top2;
                     case 2:  //TW
                       colorScale = new ColorScale(0.0, java.awt.Color.red, "0",10.0,java.awt.Color.green,"10");
                       colorScale.add(12.0,java.awt.Color.green,"12");
                       colorScale.add(20.0,java.awt.Color.red,"20");
                       scaleDescription = new String("TimeWalk (ns)");
                       break top2;
                     case 3:  //TCal
                       colorScale = new ColorScale(20.0, java.awt.Color.red, "20",24.0,java.awt.Color.green,"24");
                       colorScale.add(26.0,java.awt.Color.green,"26");
                       colorScale.add(30.0,java.awt.Color.red,"30");
                       scaleDescription = new String("TCal");
                       break top2;
                      default:
                       colorScale=null;
                       scaleDescription=null;
                       break top2;
                     }       

              default:
                colorScale=null;
               scaleDescription=null;
               break;
               }
     }
    public void setColorScale(ColorScale colorScale) {
        this.colorScale=colorScale;
        repaint();
        } 
    public ColorScale getColorScale() {
         return colorScale;
        } 
    
    // Handles the event of the user pressing down the mouse button.
    public void mousePressed(MouseEvent e){   
    }

    // Handles the event of a user dragging the mouse while holding
    // down the mouse button.
    public void mouseDragged(MouseEvent e){
    }

    // Handles the event of a user releasing the mouse button.
    public void mouseReleased(MouseEvent e){

    }
       
     // This method is required by MouseListener.
     public void mouseMoved(MouseEvent e){ 

     }

     // These methods are required by MouseMotionListener.
     public void mouseClicked(MouseEvent e){
       }

     public void mouseExited(MouseEvent e){}
     public void mouseEntered(MouseEvent e){}
                         
     // Updates the coordinates representing the location of the current rectangle.


    public void paintComponent(Graphics g){

        super.paintComponent(g);
        

        
        Graphics2D g2 = (Graphics2D)g;
	g2.setStroke(new BasicStroke(1.0f));

        if ( firstTime ) {
            dim = getSize();
            area = new Rectangle(dim);
            maxWidth = (int)dim.getWidth();
            maxHeight = (int)dim.getHeight();
            firstTime = false;
            g2.setFont(displayTextFont);
            metrics = g2.getFontMetrics();
            x = (int)(maxWidth/2.)-colorBoxWidth/2;
            y = (int)(maxHeight/2.)-colorBoxHeight/2;


        }
        // Clears the rectangle that was previously drawn.
        g2.setPaint(Color.white);
        g2.fill(area);
        
        if(colorScale!=null) colorScale.drawColorScale(g2, x, y);
        if(scaleDescription!=null) {
           g2.setFont(displayScaleFont);
           g2.setPaint(darkBlueColor);
           metrics = g2.getFontMetrics();
           String thisString = "Scale: "+scaleDescription;
           int stringWidth = metrics.stringWidth(thisString);
           int stringHeight = metrics.getHeight();
           int thisX = (int)(maxWidth/2.)-stringWidth/2;
           int thisY = (int)(maxHeight/2.)+ colorBoxHeight/2 + stringHeight + 2;
           g2.drawString(thisString,thisX,thisY);
           }


    }

 
}
