package DisplayGUI;

import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.text.*;
import java.util.regex.*;
import is.*;
import Sct.Serializable;
import Sct.IS.*;
import GuiComponents.Console.ExternalProcess;
import GuiComponents.Inspector.InspectorPanel;
import Sct_CalibrationController.TestData;
import GuiComponents.System.*;

/*
 * ISInterface.java
 *
 */

/**
 *
 * @author  robinson
 */
public class ISInterface implements InfoListener {
    
    gui guiControl;
    Hashtable serverHash;  // hash to keep track of which servers we are subscribed to...
    String theObjectName;
    Vector ISTests;        // list of current tests in IS
    String isFilterHeader;
    String isFilterRunInfo;
    TestData testData;       // the currently selected test
    String serverName;       // the currently selected server name
    int noScans;             // the number of scans in the currently selected test

    String eventServerRegex;      // regex for event server objects
    String fitServerRegex;        //regex for fit server objects
    String controlPattern;          //regex for control data
    String testServerRegex;          //regex for test server objects

    int serverIndex=0;        // the currently selected server

    public ISInterface(gui parent) {
        guiControl=parent;
//        sct = SCTInterface2.getInstance();
        is = GuiComponents.System.ISInterface.getInstance();
        serverHash = new Hashtable();
        ISTests = new Vector();
        noScans=0;
        eventServerRegex = ".*RawScanResult.*[0-9]+\\.[0-9]+(\\.[0-9]{14}|\\.SCTTestAPI_PseudoModule)?";
        fitServerRegex = ".*FitScanResult.*[0-9]+\\.[0-9]+(\\.[0-9]{14}|\\.SCTTestAPI_PseudoModule)?";
        testServerRegex = ".*Result.*[0-9]+\\.[0-9]+(\\.[0-9]{14}|\\.SCTTestAPI_PseudoModule)?";
        controlPattern = "TestData\\.[0-9]+\\.[0-9]+";

    }
        
    protected void subscribe() {
        try {
         String eventServer = SctNames.getEventISServer();
         String fitServer = SctNames.getFitISServer();
         String testServer = SctNames.getTestISServer();
         String controlServer = SctNames.getControlISServer();

         if(!serverHash.containsKey(eventServer)) {
              is.subscribe(eventServer, eventServerRegex, true, this);
              serverHash.put(eventServer, eventServerRegex);
              }
         if(!serverHash.containsKey(fitServer)) {
              is.subscribe(fitServer, fitServerRegex, true, this);
              serverHash.put(fitServer, fitServerRegex);
              }
         if(!serverHash.containsKey(testServer)) {
              is.subscribe(testServer, testServerRegex, true, this);
              serverHash.put(testServer, testServerRegex);
              }
         if(!serverHash.containsKey(controlServer)) {
             is.subscribe(controlServer, controlPattern, true, this);
             serverHash.put(controlServer, controlPattern);
             }
         String[] objects = is.getISObjects(controlServer,controlPattern);
         ISTests = new Vector();
         for (int j=0; j<objects.length; j++) {  
                   ISTests.addElement(objects[j]);
                   }  
        } catch (RuntimeException e) {
            System.err.println("Couldn't subscribe to one or more IS Server(s): " + e.getMessage());
        }
        guiControl.displayPane.setDisplayColors();
        guiControl.displayPane.refreshDisplay();   
    }

    protected void unsubscribe() {
            for(Enumeration e = serverHash.keys(); e.hasMoreElements();) {
               String server = (String)e.nextElement();
               String pattern = (String)serverHash.get(server);
               is.unsubscribe(server, pattern);
//               System.out.println(is.unsubscribe(server, pattern));
               }
            serverHash = new Hashtable();
    }

    public javax.swing.tree.DefaultMutableTreeNode getISTree() {

        String[] servers = is.getISServers();        
        
        //Setup tree
        
        javax.swing.tree.DefaultMutableTreeNode top = new javax.swing.tree.DefaultMutableTreeNode("IS Servers");
        javax.swing.tree.DefaultMutableTreeNode server;
        javax.swing.tree.DefaultMutableTreeNode object;
        for (int i=0; i<servers.length; i++) {
            is.subscribe(servers[i], this);
            server = new javax.swing.tree.DefaultMutableTreeNode(servers[i]);
            String[] objects = is.getISObjects(servers[i],".*");
            for (int j=0; j<objects.length; j++) {
                object = new javax.swing.tree.DefaultMutableTreeNode(objects[j]);
                server.add(object);
            }
            top.add(server);
        }
        return top;
//        trISServers.setModel(new DefaultTreeModel(top));
    }
    
    /** InfoListener interfaces
     */ 
    
    /**
     * Adds the new object - we must run this in the EventDispatch thread to avoid possible thread conflicts
     */
    public void infoCreated(InfoEvent e) {
       theObjectName = e.getName();
       javax.swing.SwingUtilities.invokeLater(new Runnable() {
          public void run() {
               if(theObjectName.startsWith(SctNames.getControlISServer())) { 
                    if(!ISTests.contains(theObjectName)) ISTests.addElement(theObjectName);
                    guiControl.tablesDisplayPane.updateTables();  // immediately add the control info to the table
                    }
               guiControl.displayPane.setDisplayColors();
               guiControl.displayPane.refreshDisplay();  // just refresh the display
          }
        });
    }


    
    /**
     * Object removed from IS -must run this in the EventDispatch thread to avoid possible conflicts
     */
    public void infoDeleted(InfoEvent e) {
       theObjectName = e.getName(); 
       javax.swing.SwingUtilities.invokeLater(new Runnable() {
          public void run() {   
//            String[] objectParts = theObjectName.split("\\.");
//            if(objectParts.length==5) changeStoredObject(objectParts[4],theObjectName,REMOVE_OBJECT);
             if(ISTests.contains(theObjectName)) {
                 ISTests.removeElement(theObjectName);
                 }
             guiControl.displayPane.setDisplayColors();
             guiControl.displayPane.refreshDisplay();  // just refresh the display
          }
        });
    }
    
    /**
     * Does nothing - object is still there 
     */
    public void infoUpdated(InfoEvent e) {
        theObjectName = e.getName();
        if(theObjectName.indexOf("ControlData")!=-1) guiControl.tablesDisplayPane.updateTestTable(theObjectName,ISTests.indexOf(theObjectName));
    }
//***************************************************************************
// construct a tabel vector of the current tests
   public Vector getTestTableList() {
       Vector theLine = new Vector();
       Vector tableContent = new Vector();
       theLine.addElement("Test");
       theLine.addElement("Status");
       theLine.addElement("Run");
       theLine.addElement("Scan");
       theLine.addElement("#Scans");
       tableContent.addElement(theLine);
       for(int i=0;i<ISTests.size();i++) {
           TestData thisTestData = new TestData(); 
           is.getRepository().getValue((String)ISTests.elementAt(i), thisTestData); 
           theLine=new Vector();
           if(thisTestData.testName.equals("NPtGainTest") && thisTestData.nScans==3) thisTestData.testName = "3PtGainTest";
           theLine.addElement(thisTestData.testName);
           switch(thisTestData.status) {
             case 0:
                 theLine.addElement("Executing");
                 break;
             case 1:
                 theLine.addElement("Completed");
                 break;
             case 2:
                 theLine.addElement("Aborted");
                 break;
             default:
                 theLine.addElement("Unknown");
             }
           theLine.addElement(Integer.toString(thisTestData.runNumber));
           theLine.addElement(Integer.toString(thisTestData.startScanNumber));
           theLine.addElement(Integer.toString(thisTestData.nScans));
           tableContent.addElement(theLine);
           }
       return tableContent;
    }
//***************************************************************************
   public void removeTests(int firstRowNo, int lastRowNo) {
// remove tests from IS
// firstRowNo is the first row number in the test JTable
// lastTestNo is the last
      for(int i=lastRowNo;i>=firstRowNo;i--) {
         String testObjectName = (String)ISTests.elementAt(i);
         TestData thisTestData = new TestData(); 
         is.getRepository().getValue(testObjectName, thisTestData); 
// first extract the run number, and scan numbers
         int runNo = thisTestData.runNumber;
         int startScanNumber = thisTestData.startScanNumber;
// form a regular espression
         StringBuffer regex = new StringBuffer(".*(");
         for(int k=thisTestData.startScanNumber;k<thisTestData.startScanNumber+thisTestData.nScans;k++) {
           if(k>thisTestData.startScanNumber) regex.append("|");
           regex.append("\\.");
           regex.append(Integer.toString(runNo));
           regex.append("\\.");
           regex.append(Integer.toString(k));
           regex.append("\\.");
         }
         regex.append(").*");
         String[] deletionObjects = is.getISObjects(SctNames.getEventISServer(),regex.toString());
         for(int j=0;j<deletionObjects.length;j++) System.out.println(is.remove(deletionObjects[j]));
         deletionObjects = is.getISObjects(SctNames.getFitISServer(),regex.toString());
         for(int j=0;j<deletionObjects.length;j++) System.out.println(is.remove(deletionObjects[j])); 
         deletionObjects = is.getISObjects(SctNames.getTestISServer(),regex.toString());
         for(int j=0;j<deletionObjects.length;j++) System.out.println(is.remove(deletionObjects[j]));      
// finally, the control object itself
// the object names in the control server do not have a final '.' after the first scan no 
         System.out.println(is.remove(testObjectName)); 
         ISTests.removeElementAt(i);
        }
        
     }
        public void removeObjects(Vector objectNames) {
           for(int i=0;i<objectNames.size();i++) System.out.println(is.remove((String)objectNames.elementAt(i)));
           }
        
//****************************************************************************
     public void setISFilter(int selectedTest) {
         if(selectedTest==-1) {
            testData=null;
            return;
            }
         String testObjectName = (String)ISTests.elementAt(selectedTest);
         testData = new TestData(); 
         is.getRepository().getValue(testObjectName, testData); 
         setIsFilterHeader();
         setIsFilterRunInfo();
         } 
//******************************************************************************
     public void setServerName(int selection) {
         serverIndex=selection;
         setIsFilterHeader();
          }
//**************************************************************************
      public int getNumberOfISObjects(String serialNo) {        
         int nobjects =  is.getNumberOfISObjects(getServerName(),isFilterHeader+isFilterRunInfo+serialNo);
         return nobjects;
         } 
//**************************************************************************
      public String[] getISObjects(String serialNo) {
         return is.getISObjects(getServerName(),isFilterHeader+isFilterRunInfo+serialNo);
         }
//**************************************************************************
//* return the number of scans in the currently selected test
      public int getNoScans() {
         if(testData==null) return 0;
         else return testData.nScans;
         }
//*********************************************************************
   public int getRunNo() {
         if(testData==null) return -1;
         else return testData.runNumber;
         }
   public int getScanNo() {
         if(testData==null) return -1;
         else return testData.startScanNumber;
         }
   public void launchExternalViewer(String name) {
        try {
            NameIS nameIS = new NameIS(name);
            String server = nameIS.getISServer();
            if (server.equals(SctNames.getEventISServer())) {
                launchExternalRaw(name);
                return;
            }
            
            if (server.equals(SctNames.getFitISServer())) {
                launchExternalFit(name);
                return;
            }
            
            if (name.indexOf("NPtGainTestResult") != -1) {
                launchExternalNPtGain(name);
                return;
            }
            
	    if (name.indexOf("TrimRangeTestResult") != -1) {
                launchExternalTrimRange(name);
                return;
            }
	    
            if (name.indexOf("StrobeDelayTestResult") != -1) {
                launchExternalStrobeDelay(name);
                return;
            }
            
            if (name.indexOf("NoiseOccupancyTestResult") != -1) {
                launchExternalNoiseOccupancy(name);
                return;
            }
        } catch (IllegalArgumentException iae) {            
        }
        System.out.println("Couldn't launch external viewer for: " + name );
    }
    
    public void launchExternalRaw(String name) {
        try {
            ExternalProcess p = new ExternalProcess("TestRetrieve " + name);                                    
        } catch (Exception ioe) {
            System.out.println(ioe.getMessage());
            ioe.printStackTrace();
        }
    }
    
    private void launchExternalFit(String name) {
        try {
            ExternalProcess p = new ExternalProcess("TestRetrieveFit " + name + " 0 36");                        
        } catch (Exception ioe) {
            System.out.println(ioe.getMessage());
            ioe.printStackTrace();
        }
    }
    
    private void launchExternalNPtGain(String name) {        
        try {
            ExternalProcess ep = new ExternalProcess("TestNPtGain " + name);            
        } catch (Exception ioe) {
            System.out.println(ioe.getMessage());
            ioe.printStackTrace();
        }
    }
    
    private void launchExternalStrobeDelay(String name) {
        try {
            ExternalProcess ep = new ExternalProcess("TestStrobeDelay " + name);            
        } catch (Exception ioe) {
            System.out.println(ioe.getMessage());
            ioe.printStackTrace();
        }   
    }
    
    private void launchExternalTrimRange(String name) {
        try {
            ExternalProcess ep = new ExternalProcess("TestTrimRange " + name);            
        } catch (Exception ioe) {
            System.out.println(ioe.getMessage());
            ioe.printStackTrace();
        }   
    }

    private void launchExternalNoiseOccupancy(String name) {
        try {
            ExternalProcess ep = new ExternalProcess("TestNO " + name);            
        } catch (Exception ioe) {
            System.out.println(ioe.getMessage());
            ioe.printStackTrace();
        }   
    }
    public void viewISData(String name) {
        try {
            Serializable ob = ObjectManagerIS.getInstance().readObject(name);
            InspectorPanel p = new InspectorPanel(name, ob);
            //JFrame frame = new JFrame("InspectorPanel");
            //frame.getContentPane().add(p, java.awt.BorderLayout.CENTER);
            //frame.pack();
            //frame.show();

            JFrame frame2 = new JFrame("TreeTable");                    
            frame2.getContentPane().add(new JScrollPane(p.treeTable));
            frame2.pack();
            frame2.show();

        } catch (java.io.IOException ioe) {
            System.err.println("Error reading object from IS: " + ioe + "  " +ioe.getMessage() +"\n");
        }
        System.out.println("IS Object: " + name + " plotted\n");
    }
    private void setIsFilterHeader() {
         switch(serverIndex) {
            case 0:
                isFilterHeader = "SctData::RawScanResult";
               break;
            case 1:
                isFilterHeader = "SctData::FitScanResult";
               break;
            case 2:
            default:
               isFilterHeader="SctData::"+testData.testName+"Result";
               break;
            } 
     }
    private void setIsFilterRunInfo() {
         StringBuffer regex = new StringBuffer("\\."+Integer.toString(testData.runNumber)+"\\.(");
         for(int k=testData.startScanNumber;k<testData.startScanNumber+testData.nScans;k++) {
           if(k>testData.startScanNumber) regex.append("|");
           regex.append(Integer.toString(k));
           regex.append("\\.");
         }
         regex.append(").*"); 
         isFilterRunInfo=regex.toString();
         }
    private String getServerName() {
         String serverName=SctNames.getEventISServer();
         switch(serverIndex) {
            case 0:
               serverName=SctNames.getEventISServer();
               break;
            case 1:
               serverName=SctNames.getFitISServer();
               break;
            case 2:
            default:
               serverName=SctNames.getTestISServer();
               break;
            }
          return serverName;
          }


     private GuiComponents.System.ISInterface is;


}
